import datetime
import logging
from PyQt5.QtCore import QSize, Qt,  pyqtSignal, pyqtSlot
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QApplication, QDialog,  QMenu, QMessageBox,  QVBoxLayout
from xulpymoney.ui.Ui_frmInvestmentReport import Ui_frmInvestmentReport
from xulpymoney.ui.frmInvestmentOperationsAdd import frmInvestmentOperationsAdd
from xulpymoney.ui.frmDividendsAdd import frmDividendsAdd
from xulpymoney.ui.frmSellingPoint import frmSellingPoint
from xulpymoney.ui.frmQuotesIBM import frmQuotesIBM
from xulpymoney.ui.wdgDisReinvest import wdgDisReinvest
from xulpymoney.ui.frmSharesTransfer import frmSharesTransfer
from xulpymoney.ui.frmHlContractAdd import frmHlContractAdd
from xulpymoney.ui.frmSplit import frmSplit
from xulpymoney.libxulpymoney import Investment, Money, Percentage, DividendHomogeneusManager, InvestmentOperationHomogeneusManager, days2string, HlContractManagerHomogeneus
from xulpymoney.libxulpymoneytypes import eMoneyCurrency

class frmInvestmentReport(QDialog, Ui_frmInvestmentReport):
    frmInvestmentOperationsAdd_initiated=pyqtSignal(frmInvestmentOperationsAdd)#Se usa para cargar datos de ordenes en los datos de este formulario
    def __init__(self, mem, inversion=None,  parent=None):
        """Accounts es un set cuentas
        TIPOS DE ENTRADAS:        
         1  Inserción de Opercuentas
         2  Inversion=x"""
        QDialog.__init__(self, parent)
        QApplication.setOverrideCursor(Qt.WaitCursor)
        self.hide()
        self.setupUi(self)
        self.mem=mem
        self.investment=inversion
        
        self.selDividend=None#Dividend seleccionado
        
        #arrays asociados a tablas
        self.op=None#Sera un SetInvestmentOperations
         
        self.ise.setupUi(self.mem,  self.investment)
        
        self.dividends=DividendHomogeneusManager(self.mem, self.investment)
        
        self.tblDividends.settings(self.mem, "frmInvestmentReport")         
        self.tblDividendsAccountCurrency.settings(self.mem, "frmInvestmentReport")
        self.tblInvestmentCurrent.settings(self.mem, "frmInvestmentReport")
        self.tblInvestmentCurrentAccountCurrency.settings(self.mem, "frmInvestmentReport")
        self.tblOperations.settings(self.mem, "frmInvestmentReport")
        self.tblOperationsAccountCurrency.settings(self.mem, "frmInvestmentReport")
        self.tblInvestmentHistorical.settings(self.mem, "frmInvestmentReport")
        self.tblInvestmentHistoricalAccountCurrency.settings(self.mem,  "frmInvestmentReport")
        self.tblHlContracts.settings(self.mem, "frmInvestmentReport")
        self.ise.cmd.released.connect(self.on_cmdISE_released)
        self.mem.data.accounts_active().qcombobox(self.cmbAccount)
        
        if self.investment==None:
            self.tipo=1
            self.cmdInvestment.setText(self.tr("Add a new investment"))
            self.lblTitulo.setText(self.tr("New investment"))
            self.investment=None
            self.tab.setCurrentIndex(0)
            self.tabDividends.setEnabled(False)
            self.tabOperacionesHistoricas.setEnabled(False)
            self.tabInvestmentCurrent.setEnabled(False)
            self.ise.setSelected(None)
            self.cmdPuntoVenta.setEnabled(False)
        else:
            self.tipo=2    
            self.tab.setCurrentIndex(1)
            self.lblTitulo.setText(self.investment.name)
            self.txtInvestment.setText(self.investment.name)
            self.txtVenta.setText(self.investment.venta)
            if self.investment.selling_expiration==None:
                self.chkExpiration.setCheckState(Qt.Unchecked)
            else:
                self.chkExpiration.setCheckState(Qt.Checked)
                self.calExpiration.setSelectedDate(self.investment.selling_expiration)
            self.ise.setSelected(self.investment.product)
            self.cmdPuntoVenta.setEnabled(True)
            self.cmbAccount.setCurrentIndex(self.cmbAccount.findData(self.investment.account.id))
            self.update_tables()      
            if len(self.op.arr)!=0 or len(self.dividends.arr)!=0:#CmbAccount está desabilitado si hay dividends o operinversiones
                self.cmbAccount.setEnabled(False)  

            #Removes contract tab when it isn't neccessary
            if self.investment.product.high_low==False:
                self.tab.removeTab(5)

        self.cmdInvestment.setEnabled(False)    
        self.showMaximized()
        
        
        
        QApplication.restoreOverrideCursor()

    def load_tabDividends(self):        
        (sumneto, sumbruto, sumretencion, sumcomision)=self.dividends.myqtablewidget(self.tblDividends)
        if self.investment.account.currency==self.investment.product.currency:
            self.grpDividendsAccountCurrency.hide()
        else:
            self.dividends.myqtablewidget(self.tblDividendsAccountCurrency, type=2)
        if self.chkHistoricalDividends.checkState()==Qt.Unchecked:
            estimacion=self.investment.product.estimations_dps.currentYear()
            if estimacion.estimation!=None:
                acciones=self.investment.shares()
                tpccalculado=Percentage(estimacion.estimation, self.investment.product.result.basic.last.quote)
                self.lblDivFechaRevision.setText(self.tr('Estimation review date: {0}').format(estimacion.date_estimation))
                self.lblDivAnualEstimado.setText(self.tr("Estimated annual dividend is {0} ({1} per share)").format(tpccalculado,  self.investment.product.currency.string(estimacion.estimation)))
                self.lblDivSaldoEstimado.setText(self.tr("Estimated balance: {0} ({1} after taxes)").format( self.investment.product.currency.string(acciones*estimacion.estimation),  self.investment.product.currency.string(acciones*estimacion.estimation*(1-self.mem.dividendwithholding))))
            self.lblDivTPC.setText(self.tr("% Invested: {}").format(self.dividends.percentage_from_invested(type=1)))
            self.lblDivTAE.setText(self.tr("% APR from invested: {}").format(self.dividends.percentage_tae_from_invested(type=1)))
            self.grpDividendsEstimation.show()
            self.grpDividendsEfectivos.show()
        else:
            self.grpDividendsEstimation.hide()
            self.grpDividendsEfectivos.hide()
       
    def on_chkOperaciones_stateChanged(self, state):
        if state==Qt.Unchecked:
            first=self.investment.op_actual.first()
            if first==None:
                dt=self.mem.localzone.now()
            else:
                dt=first.datetime
            self.op=self.investment.op.ObjectManager_from_datetime(dt, self.mem, self.investment)
        else:
            self.op=self.investment.op
            
        self.op.selected=None
        self.op.myqtablewidget(self.tblOperations)
        
        if self.investment.product.currency==self.investment.account.currency:#Multidivisa
            self.grpOperationsAccountCurrency.hide()
        else:
            self.op.myqtablewidget(self.tblOperationsAccountCurrency, type=2)

    def update_tables(self):             
        #Actualiza el indice de referencia porque ha cambiado
        self.investment.op_actual.get_valor_benchmark(self.mem.data.benchmark)
        
        if self.investment.merge==0:
            self.on_chkOperaciones_stateChanged(self.chkOperaciones.checkState())
            self.chkOperaciones.setEnabled(True)
        else:#merge 1 y 2
            self.chkOperaciones.setChecked(Qt.Checked)
            self.chkOperaciones.setEnabled(False)
        
        self.investment.op_actual.myqtablewidget(self.tblInvestmentCurrent, self.investment.product.result.basic.last, type=1)
        self.investment.op_historica.myqtablewidget(self.tblInvestmentHistorical,  type=1 )
        if self.investment.product.currency==self.investment.account.currency:#Multidivisa
            self.grpCurrentAccountCurrency.hide()
            self.grpHistoricalAccountCurrency.hide()
        else:
            m=Money(self.mem, 1, self.investment.account.currency)
            
            self.grpCurrentAccountCurrency.setTitle(self.tr("Current status in account currency ( {} = {} at {} )").format(m.string(6), m.convert(self.investment.product.currency, self.mem.localzone.now()).string(6), m.conversionDatetime(self.investment.product.currency, self.mem.localzone.now())))
            self.investment.op_actual.myqtablewidget(self.tblInvestmentCurrentAccountCurrency, self.investment.product.result.basic.last,  type=2)
            self.investment.op_historica.myqtablewidget(self.tblInvestmentHistoricalAccountCurrency, type=2 )
        
        self.lblAge.setText(self.tr("Current operations average age: {0}".format(days2string(self.investment.op_actual.average_age()))))
        
        if self.investment!=None:#We are adding a new investment
            if self.investment.merge==0:
                self.on_chkHistoricalDividends_stateChanged(self.chkHistoricalDividends.checkState())
                self.chkHistoricalDividends.setEnabled(True)
            else:
                self.chkHistoricalDividends.setChecked(Qt.Checked)
                self.chkHistoricalDividends.setEnabled(False)
        
        #Show contracts if it's a hig_low product
        if self.investment.product.high_low==True:
            self.on_chkHistoricalContracts_stateChanged(self.chkHistoricalContracts.checkState())

    @pyqtSlot() 
    def on_actionDividendAdd_triggered(self):
        w=frmDividendsAdd(self.mem, self.investment,  None)
        w.exec_()
        self.on_chkHistoricalDividends_stateChanged(self.chkHistoricalDividends.checkState())

        
    @pyqtSlot() 
    def on_actionDividendEdit_triggered(self):
        w=frmDividendsAdd(self.mem, self.investment, self.selDividend)
        w.exec_()
        self.on_chkHistoricalDividends_stateChanged(self.chkHistoricalDividends.checkState())

        
    @pyqtSlot() 
    def on_actionDividendRemove_triggered(self):
        self.selDividend.borrar()
        self.mem.con.commit()
        self.on_chkHistoricalDividends_stateChanged(self.chkHistoricalDividends.checkState())

    @pyqtSlot() 
    def on_actionContractAdd_triggered(self):
        w=frmHlContractAdd(self.mem, self.investment,  None, self)
        w.exec_()
        self.on_chkHistoricalContracts_stateChanged(self.chkHistoricalContracts.checkState())

        
    @pyqtSlot() 
    def on_actionContractEdit_triggered(self):
        w=frmHlContractAdd(self.mem, self.investment, self.hlcontracts.selected, self)
        w.exec_()
        self.on_chkHistoricalContracts_stateChanged(self.chkHistoricalContracts.checkState())

        
    @pyqtSlot() 
    def on_actionContractDelete_triggered(self):
        self.investment.hlcontractmanager.delete_from_db(self.hlcontracts.selected)
        self.mem.con.commit()
        self.on_chkHistoricalContracts_stateChanged(self.chkHistoricalContracts.checkState())

    @pyqtSlot() 
    def on_actionChangeBenchmarkPrice_triggered(self):
        w=frmQuotesIBM(self.mem, self.mem.data.benchmark, self.investment.op_actual.selected.referenciaindice, self)
        w.txtQuote.setFocus()
        w.exec_() 
        self.update_tables()

    @pyqtSlot() 
    def on_actionDisReinvest_triggered(self):
        d=QDialog()       
        d.resize(self.mem.settings.value("frmInvestmentReport/qdialog_disreinvest", QSize(1024, 768)))
        d.setWindowTitle(self.tr("Divest / Reinvest simulation"))
        w=wdgDisReinvest(self.mem, self.investment, False, d)
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_()
        self.mem.settings.setValue("frmInvestmentReport/qdialog_disreinvest", d.size())
        
    @pyqtSlot() 
    def on_actionDisReinvestProduct_triggered(self):
        d=QDialog()       
        d.resize(self.mem.settings.value("frmInvestmentReport/qdialog_disreinvest", QSize(1024, 768)))
        d.setWindowTitle(self.tr("Divest / Reinvest simulation"))
        w=wdgDisReinvest(self.mem, self.investment, True,  d)
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_()
        self.mem.settings.setValue("frmInvestmentReport/qdialog_disreinvest", d.size())
        
    @pyqtSlot() 
    def on_actionOperationAdd_triggered(self):
        if self.investment.product.result.basic.last.quote==None:
            m=QMessageBox()
            m.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
            m.setIcon(QMessageBox.Information)
            m.setText(self.tr("Before adding a operation, you must add the current price of the product."))
            m.exec_()    
            w=frmQuotesIBM(self.mem,  self.investment.product)
            w.exec_()   
            if w.result()==QDialog.Accepted:
                self.investment.product.result.basic.load_from_db()
            else:
                return
            
        w=frmInvestmentOperationsAdd(self.mem, self.investment, None, self)
        self.frmInvestmentOperationsAdd_initiated.emit(w)
        w.exec_()
        
        #if num shares after add is 0, changes expiration date to today-1
        if self.investment.shares()==0:
            self.calExpiration.setSelectedDate(datetime.date.today()-datetime.timedelta(days=1))
            self.on_cmdInvestment_released()
            
        self.update_tables()    
        
    @pyqtSlot() 
    def on_actionOperationEdit_triggered(self):
        w=frmInvestmentOperationsAdd(self.mem, self.investment, self.op.selected, self)
        w.exec_()
        self.update_tables() 
        
        #if num shares after add is 0, changes expiration date to today-1
        if self.investment.shares()==0:
            self.calExpiration.setSelectedDate(datetime.date.today()-datetime.timedelta(days=1))
            self.on_cmdInvestment_released()

    @pyqtSlot() 
    def on_actionSplit_triggered(self):
        w=frmSplit(self.mem, self.investment.product)
        w.exec_()   
        if w.result()==QDialog.Accepted:
            self.update_tables()
        
    @pyqtSlot() 
    def on_actionSharesTransfer_triggered(self):
        w=frmSharesTransfer(self.mem, self.investment, self)
        w.exec_()
        
        #if num shares after add is 0, changes expiration date to today-1
        if self.investment.shares()==0:
            self.calExpiration.setSelectedDate(datetime.date.today()-datetime.timedelta(days=1))
            self.on_cmdInvestment_released()
        
        self.update_tables()                 
    
    @pyqtSlot() 
    def on_actionRangeReport_triggered(self):
        self.op.selected.show_in_ranges= not self.op.selected.show_in_ranges
        self.op.selected.save()
        self.mem.con.commit()
        #self.op doesn't belong to self.mem.data, it's a set of this widget, so I need to reload investment of the self.mem.data
        self.mem.data.investments.find_by_id(self.investment.id).get_operinversiones()
        self.update_tables()

    @pyqtSlot() 
    def on_actionSharesTransferUndo_triggered(self):
        if self.mem.data.investments_active().traspaso_valores_deshacer(self.op.selected)==False:
            m=QMessageBox()
            m.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
            m.setIcon(QMessageBox.Information)
            m.setText(self.tr("Shares transfer couldn't be done."))
            m.exec_()          
            return
        self.update_tables()       

    @pyqtSlot() 
    def on_cmdPuntoVenta_released(self):
        f=frmSellingPoint(self.mem, self.investment)
        f.txtPrice.setText(self.txtVenta.text())
        f.exec_()
        self.txtVenta.setText(str(f.puntoventa.round(6)))

    @pyqtSlot() 
    def on_actionOperationDelete_triggered(self):
        self.investment.op.remove(self.op.selected)#debe borrarse de self.investment.op, no de self.op, ya qque self.update_tables reescribe clone_from_datetime
        self.mem.con.commit()     
        
        #if num shares after add is 0, changes expiration date to today-1
        if self.investment.shares()==0:
            self.calExpiration.setSelectedDate(datetime.date.today()-datetime.timedelta(days=1))
            self.on_cmdInvestment_released()
        
        self.update_tables()

        
        
    def on_chkHistoricalDividends_stateChanged(self, state):
        self.tblDividends.clearSelection()
        self.tblDividendsAccountCurrency.clearSelection()
        self.selDividend=None        
        if self.investment.merge==0:
            if state==Qt.Unchecked:   
                self.dividends=self.investment.setDividends_from_current_operations()
            elif state==Qt.Checked:
                self.dividends=self.investment.setDividends_from_operations()
            else:
                self.dividends.clean()
        elif self.investment.merge==1:
            self.dividends=self.mem.data.investments_active().setDividends_merging_current_operation_dividends(self.investment.product)
        elif self.investment.merge==2:
            self.dividends=self.mem.data.investments.setDividends_merging_operation_dividends(self.investment.product)
        self.load_tabDividends()

    def on_cmdISE_released(self):
        if self.investment==None or self.investment.merge==0:
            self.cmdInvestment.setEnabled(True)

    def on_txtVenta_textChanged(self):
        if self.investment==None or self.investment.merge==0:
            self.cmdInvestment.setEnabled(True)

    def on_txtInvestment_textChanged(self):
        if self.investment==None or self.investment.merge==0:
            self.cmdInvestment.setEnabled(True)

    def on_cmbTipoInvestment_currentIndexChanged(self, index):
        if self.investment==None or self.investment.merge==0:
            self.cmdInvestment.setEnabled(True)

    def on_calExpiration_selectionChanged(self):
        if self.investment==None or self.investment.merge==0:
            self.cmdInvestment.setEnabled(True)

    def on_chkExpiration_stateChanged(self, state):
        if self.investment==None or self.investment.merge==0:
            self.cmdInvestment.setEnabled(True)

    def on_cmdToday_released(self):
        self.calExpiration.setSelectedDate(datetime.date.today())

    def on_cmdInvestment_released(self):
        if self.ise.selected==None:
            m=QMessageBox()
            m.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
            m.setIcon(QMessageBox.Information)
            m.setText(self.tr("You must select a product to continue."))
            m.exec_()     
            return
        inversion=self.txtInvestment.text()
        venta=self.txtVenta.decimal()
        id_cuentas=int(self.cmbAccount.itemData(self.cmbAccount.currentIndex()))
        product=self.ise.selected
        if self.chkExpiration.checkState()==Qt.Unchecked:
            expiration=None
        else:
            expiration=self.calExpiration.selectedDate().toPyDate()

        if self.mem.data.products.find_by_id(product.id)==None:
            print ("Cargando otro producto en mem")
            product.estimations_dps.load_from_db()
            product.result.basic.load_from_db()
            self.mem.data.products.append(product)        

        if self.tipo==1:        #insertar
            self.investment=Investment(self.mem).init__create(inversion,   venta,  self.mem.data.accounts_active().find_by_id(id_cuentas), product, expiration, True)      
            self.investment.save()
            self.mem.con.commit()    
            #Lo añade con las operaciones vacias pero calculadas.
            self.investment.op=InvestmentOperationHomogeneusManager(self.mem, self.investment)
            (self.investment.op_actual, self.investment.op_historica)=self.investment.op.get_current_and_historical_operations()
            self.mem.data.investments.append(self.investment)
            self.done(0)
        elif self.tipo==2:
            self.investment.name=inversion
            self.investment.venta=venta
            self.investment.product=product
            self.investment.selling_expiration=expiration
            self.investment.save()##El id y el id_cuentas no se pueden modificar
            self.mem.con.commit()
            self.cmdInvestment.setEnabled(False)
        
    def on_tblOperations_customContextMenuRequested(self,  pos):
        if self.investment.qmessagebox_inactive() or self.investment.account.qmessagebox_inactive()or self.investment.account.eb.qmessagebox_inactive():
            return
        
        if self.op.selected==None:
            self.actionOperationDelete.setEnabled(False)
            self.actionOperationEdit.setEnabled(False)
            self.actionRangeReport.setEnabled(False)
        else:
            if self.op.selected.tipooperacion.id==10:#Traspaso valores destino
                self.actionOperationDelete.setEnabled(False)
                self.actionOperationEdit.setEnabled(False)
            else:
                self.actionOperationDelete.setEnabled(True)
                self.actionOperationEdit.setEnabled(True)
            self.actionRangeReport.setEnabled(True)


        if self.investment.merge!=0:
            self.actionOperationAdd.setEnabled(False)
            self.actionOperationDelete.setEnabled(False)
            self.actionOperationEdit.setEnabled(False)
            self.actionRangeReport.setEnabled(False)
            self.actionSplit.setEnabled(False)


        menu=QMenu()
        menu.addAction(self.actionOperationAdd)
        
        menu.addAction(self.actionOperationEdit)
        menu.addAction(self.actionOperationDelete)
        
        if self.op.selected!=None:
            if self.op.selected.tipooperacion.id==9:#Traspaso valores origen
                menu.addSeparator()
                menu.addAction(self.actionSharesTransferUndo)
                
        menu.addSeparator()
        menu.addAction(self.actionRangeReport)
        menu.addSeparator()
        menu.addAction(self.actionSplit)
        menu.exec_(self.tblOperations.mapToGlobal(pos))


    def on_tblInvestmentCurrent_itemSelectionChanged(self):
        self.investment.op_actual.selected=None
        try:
            for i in self.tblInvestmentCurrent.selectedItems():#itera por cada item no row.
                if i.row()<self.investment.op_actual.length():#Due to total file
                    self.investment.op_actual.selected=self.investment.op_actual.arr[i.row()]
        except:
            self.investment.op_actual.selected=None
        print (self.tr("Selected: {0}".format(str(self.investment.op_actual.selected))))
        
        
    def on_tblInvestmentCurrent_customContextMenuRequested(self,  pos):
        
        if self.investment.qmessagebox_inactive() or self.investment.account.qmessagebox_inactive() or self.investment.account.eb.qmessagebox_inactive():
            return
                
        if self.investment.op_actual.selected:
            self.actionChangeBenchmarkPrice.setEnabled(True)
        else:
            self.actionChangeBenchmarkPrice.setEnabled(False)
            
        if self.investment.merge!=0:
            self.actionOperationAdd.setEnabled(False)
            self.actionChangeBenchmarkPrice.setEnabled(False)
            self.actionSharesTransfer.setEnabled(False)
            
        menu=QMenu()
        menu.addAction(self.actionDisReinvest)
        menu.addAction(self.actionDisReinvestProduct)
        menu.addSeparator()
        menu.addAction(self.actionOperationAdd)
        menu.addSeparator()
        menu.addAction(self.actionSharesTransfer)
        menu.addSeparator()
        menu.addAction(self.actionChangeBenchmarkPrice)
        
        
        menu.exec_(self.tblInvestmentCurrent.mapToGlobal(pos))

    def on_tblOperations_itemSelectionChanged(self):
        self.op.selected=None
        try:
            for i in self.tblOperations.selectedItems():#itera por cada item no row.
                self.op.selected=self.op.arr[i.row()]
                if self.op.selected.show_in_ranges==True:
                    self.actionRangeReport.setText(self.tr("Hide in range report"))
                    self.actionRangeReport.setIcon(QIcon(":/xulpymoney/eye_red.png"))
                else:
                    self.actionRangeReport.setText(self.tr("Show in range report"))
                    self.actionRangeReport.setIcon(QIcon(":/xulpymoney/eye.png"))
        except:
            self.op.selected=None
        print (self.tr("Selected: {0}".format(str(self.op.selected))))

    def on_tblDividends_customContextMenuRequested(self,  pos):
        if self.investment.qmessagebox_inactive() or self.investment.account.qmessagebox_inactive() or self.investment.account.eb.qmessagebox_inactive():
            return

        if self.selDividend==None:
            self.actionDividendRemove.setEnabled(False)
            self.actionDividendEdit.setEnabled(False)
        else:
            self.actionDividendRemove.setEnabled(True)
            self.actionDividendEdit.setEnabled(True)
            
        if self.investment.merge!=0:
            self.actionDividendAdd.setEnabled(False)
            self.actionDividendRemove.setEnabled(False)
            self.actionDividendEdit.setEnabled(False)
            
        menu=QMenu()
        menu.addAction(self.actionDividendAdd)
        menu.addAction(self.actionDividendRemove)
        menu.addAction(self.actionDividendEdit)
        menu.exec_(self.tblDividends.mapToGlobal(pos))

    def on_tblDividends_itemSelectionChanged(self):
        try:
            for i in self.tblDividends.selectedItems():#itera por cada item no rowse.
                self.selDividend=self.dividends.arr[i.row()]
        except:
            self.selDividend=None
        print ("Dividend selected: " +  str(self.selDividend))        


    def on_chkHistoricalContracts_stateChanged(self, state):
        self.tblHlContracts.clearSelection()
        if state==Qt.Unchecked:
            if self.investment.op_actual.length()>0:
                self.hlcontracts=self.investment.hlcontractmanager.ObjectManager_from_datetime (self.investment.op_actual.first().datetime, self.mem, self.investment)
            else:
                self.hlcontracts=HlContractManagerHomogeneus(self.mem, self.investment)
        else:
            self.hlcontracts=self.investment.hlcontractmanager
            
        self.hlcontracts.myqtablewidget(self.tblHlContracts, eMoneyCurrency.Product)

    def on_tblHlContracts_itemSelectionChanged(self):
        try:
            for i in self.tblHlContracts.selectedItems():#itera por cada item no row.
                self.hlcontracts.selected=self.hlcontracts.arr[i.row()]
        except:
            self.hlcontracts.cleanSelection()
        logging.debug(self.tr("Selected in tblHlContracts: {0}".format(str(self.hlcontracts.selected))))

    def on_tblHlContracts_customContextMenuRequested(self,  pos):
        if self.hlcontracts.selected:
            self.actionContractEdit.setEnabled(True)
            self.actionContractDelete.setEnabled(True)
        else:
            self.actionContractEdit.setEnabled(False)
            self.actionContractDelete.setEnabled(False)
            
        menu=QMenu()
        menu.addAction(self.actionContractAdd)
        menu.addAction(self.actionContractEdit)
        menu.addAction(self.actionContractDelete)       
        
        menu.exec_(self.tblHlContracts.mapToGlobal(pos))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmAccess.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmAccess(object):
    def setupUi(self, frmAccess):
        frmAccess.setObjectName("frmAccess")
        frmAccess.setWindowModality(QtCore.Qt.WindowModal)
        frmAccess.resize(340, 409)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(frmAccess.sizePolicy().hasHeightForWidth())
        frmAccess.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/kmplot.jpg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmAccess.setWindowIcon(icon)
        frmAccess.setModal(True)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(frmAccess)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.lblPixmap = QtWidgets.QLabel(frmAccess)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblPixmap.sizePolicy().hasHeightForWidth())
        self.lblPixmap.setSizePolicy(sizePolicy)
        self.lblPixmap.setMinimumSize(QtCore.QSize(80, 80))
        self.lblPixmap.setMaximumSize(QtCore.QSize(80, 80))
        self.lblPixmap.setText("")
        self.lblPixmap.setPixmap(QtGui.QPixmap(":/xulpymoney/keki.png"))
        self.lblPixmap.setScaledContents(True)
        self.lblPixmap.setAlignment(QtCore.Qt.AlignCenter)
        self.lblPixmap.setObjectName("lblPixmap")
        self.horizontalLayout_2.addWidget(self.lblPixmap)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.lbl = QtWidgets.QLabel(frmAccess)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setText("")
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout.addWidget(self.lbl)
        self.line = QtWidgets.QFrame(frmAccess)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lblLanguage = QtWidgets.QLabel(frmAccess)
        self.lblLanguage.setObjectName("lblLanguage")
        self.horizontalLayout.addWidget(self.lblLanguage)
        self.cmbLanguages = QtWidgets.QComboBox(frmAccess)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbLanguages.sizePolicy().hasHeightForWidth())
        self.cmbLanguages.setSizePolicy(sizePolicy)
        self.cmbLanguages.setMaximumSize(QtCore.QSize(200, 16777215))
        self.cmbLanguages.setFocusPolicy(QtCore.Qt.NoFocus)
        self.cmbLanguages.setObjectName("cmbLanguages")
        self.horizontalLayout.addWidget(self.cmbLanguages)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.lblServer = QtWidgets.QLabel(frmAccess)
        self.lblServer.setObjectName("lblServer")
        self.horizontalLayout_3.addWidget(self.lblServer)
        self.txtServer = QtWidgets.QLineEdit(frmAccess)
        self.txtServer.setMaximumSize(QtCore.QSize(200, 16777215))
        self.txtServer.setInputMethodHints(QtCore.Qt.ImhNone)
        self.txtServer.setText("")
        self.txtServer.setEchoMode(QtWidgets.QLineEdit.Normal)
        self.txtServer.setObjectName("txtServer")
        self.horizontalLayout_3.addWidget(self.txtServer)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.lblPort = QtWidgets.QLabel(frmAccess)
        self.lblPort.setObjectName("lblPort")
        self.horizontalLayout_4.addWidget(self.lblPort)
        self.txtPort = QtWidgets.QLineEdit(frmAccess)
        self.txtPort.setMaximumSize(QtCore.QSize(200, 16777215))
        self.txtPort.setText("")
        self.txtPort.setObjectName("txtPort")
        self.horizontalLayout_4.addWidget(self.txtPort)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.lblDatabase = QtWidgets.QLabel(frmAccess)
        self.lblDatabase.setObjectName("lblDatabase")
        self.horizontalLayout_5.addWidget(self.lblDatabase)
        self.txtDB = QtWidgets.QLineEdit(frmAccess)
        self.txtDB.setMaximumSize(QtCore.QSize(200, 16777215))
        self.txtDB.setObjectName("txtDB")
        self.horizontalLayout_5.addWidget(self.txtDB)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.lblUser = QtWidgets.QLabel(frmAccess)
        self.lblUser.setObjectName("lblUser")
        self.horizontalLayout_6.addWidget(self.lblUser)
        self.txtUser = QtWidgets.QLineEdit(frmAccess)
        self.txtUser.setMaximumSize(QtCore.QSize(200, 16777215))
        self.txtUser.setObjectName("txtUser")
        self.horizontalLayout_6.addWidget(self.txtUser)
        self.verticalLayout.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.lblPass = QtWidgets.QLabel(frmAccess)
        self.lblPass.setObjectName("lblPass")
        self.horizontalLayout_7.addWidget(self.lblPass)
        self.txtPass = QtWidgets.QLineEdit(frmAccess)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtPass.sizePolicy().hasHeightForWidth())
        self.txtPass.setSizePolicy(sizePolicy)
        self.txtPass.setMaximumSize(QtCore.QSize(200, 16777215))
        self.txtPass.setInputMethodHints(QtCore.Qt.ImhHiddenText|QtCore.Qt.ImhNoAutoUppercase|QtCore.Qt.ImhNoPredictiveText|QtCore.Qt.ImhSensitiveData)
        self.txtPass.setEchoMode(QtWidgets.QLineEdit.Password)
        self.txtPass.setObjectName("txtPass")
        self.horizontalLayout_7.addWidget(self.txtPass)
        self.verticalLayout.addLayout(self.horizontalLayout_7)
        self.cmdYN = QtWidgets.QDialogButtonBox(frmAccess)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmdYN.sizePolicy().hasHeightForWidth())
        self.cmdYN.setSizePolicy(sizePolicy)
        self.cmdYN.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedKingdom))
        self.cmdYN.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.cmdYN.setCenterButtons(True)
        self.cmdYN.setObjectName("cmdYN")
        self.verticalLayout.addWidget(self.cmdYN)
        self.horizontalLayout_8.addLayout(self.verticalLayout)

        self.retranslateUi(frmAccess)
        QtCore.QMetaObject.connectSlotsByName(frmAccess)
        frmAccess.setTabOrder(self.txtPass, self.cmdYN)
        frmAccess.setTabOrder(self.cmdYN, self.txtServer)
        frmAccess.setTabOrder(self.txtServer, self.txtPort)
        frmAccess.setTabOrder(self.txtPort, self.txtDB)
        frmAccess.setTabOrder(self.txtDB, self.txtUser)

    def retranslateUi(self, frmAccess):
        _translate = QtCore.QCoreApplication.translate
        self.lblLanguage.setText(_translate("frmAccess", "Language"))
        self.lblServer.setText(_translate("frmAccess", "Server"))
        self.lblPort.setText(_translate("frmAccess", "Port"))
        self.lblDatabase.setText(_translate("frmAccess", "Database"))
        self.lblUser.setText(_translate("frmAccess", "User"))
        self.lblPass.setText(_translate("frmAccess", "Password"))


import xulpymoney.images.xulpymoney_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgConceptsHistorical.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgConceptsHistorical(object):
    def setupUi(self, wdgConceptsHistorical):
        wdgConceptsHistorical.setObjectName("wdgConceptsHistorical")
        wdgConceptsHistorical.resize(1280, 612)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/history.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        wdgConceptsHistorical.setWindowIcon(icon)
        self.horizontalLayout = QtWidgets.QHBoxLayout(wdgConceptsHistorical)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tab = QtWidgets.QTabWidget(wdgConceptsHistorical)
        self.tab.setTabsClosable(True)
        self.tab.setObjectName("tab")
        self.widget = QtWidgets.QWidget()
        self.widget.setObjectName("widget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.table = myQTableWidget(self.widget)
        self.table.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.table.setAlternatingRowColors(True)
        self.table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.table.setObjectName("table")
        self.table.setColumnCount(14)
        self.table.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(8, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(9, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(10, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(11, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(12, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(13, item)
        self.table.verticalHeader().setVisible(False)
        self.horizontalLayout_2.addWidget(self.table)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/hucha.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tab.addTab(self.widget, icon1, "")
        self.verticalLayout.addWidget(self.tab)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.actionShowMonth = QtWidgets.QAction(wdgConceptsHistorical)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpymoney/search.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowMonth.setIcon(icon2)
        self.actionShowMonth.setObjectName("actionShowMonth")
        self.actionShowYear = QtWidgets.QAction(wdgConceptsHistorical)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/xulpymoney/document-preview-archive.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowYear.setIcon(icon3)
        self.actionShowYear.setObjectName("actionShowYear")

        self.retranslateUi(wdgConceptsHistorical)
        self.tab.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(wdgConceptsHistorical)

    def retranslateUi(self, wdgConceptsHistorical):
        _translate = QtCore.QCoreApplication.translate
        wdgConceptsHistorical.setWindowTitle(_translate("wdgConceptsHistorical", "Historical concepts report"))
        item = self.table.horizontalHeaderItem(0)
        item.setText(_translate("wdgConceptsHistorical", "Year"))
        item = self.table.horizontalHeaderItem(1)
        item.setText(_translate("wdgConceptsHistorical", "January"))
        item = self.table.horizontalHeaderItem(2)
        item.setText(_translate("wdgConceptsHistorical", "February"))
        item = self.table.horizontalHeaderItem(3)
        item.setText(_translate("wdgConceptsHistorical", "March"))
        item = self.table.horizontalHeaderItem(4)
        item.setText(_translate("wdgConceptsHistorical", "April"))
        item = self.table.horizontalHeaderItem(5)
        item.setText(_translate("wdgConceptsHistorical", "May"))
        item = self.table.horizontalHeaderItem(6)
        item.setText(_translate("wdgConceptsHistorical", "June"))
        item = self.table.horizontalHeaderItem(7)
        item.setText(_translate("wdgConceptsHistorical", "July"))
        item = self.table.horizontalHeaderItem(8)
        item.setText(_translate("wdgConceptsHistorical", "August"))
        item = self.table.horizontalHeaderItem(9)
        item.setText(_translate("wdgConceptsHistorical", "September"))
        item = self.table.horizontalHeaderItem(10)
        item.setText(_translate("wdgConceptsHistorical", "October"))
        item = self.table.horizontalHeaderItem(11)
        item.setText(_translate("wdgConceptsHistorical", "November"))
        item = self.table.horizontalHeaderItem(12)
        item.setText(_translate("wdgConceptsHistorical", "December"))
        item = self.table.horizontalHeaderItem(13)
        item.setText(_translate("wdgConceptsHistorical", "Total"))
        self.tab.setTabText(self.tab.indexOf(self.widget), _translate("wdgConceptsHistorical", "Historical report"))
        self.actionShowMonth.setText(_translate("wdgConceptsHistorical", "Show month operations"))
        self.actionShowMonth.setToolTip(_translate("wdgConceptsHistorical", "Show month operations"))
        self.actionShowYear.setText(_translate("wdgConceptsHistorical", "Show year operations"))
        self.actionShowYear.setToolTip(_translate("wdgConceptsHistorical", "Show year operations"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
import xulpymoney.images.xulpymoney_rc



import datetime
from PyQt5.QtCore import pyqtSlot, QSize
from PyQt5.QtWidgets import QWidget, QDialog, QVBoxLayout, QMenu, QMessageBox
from xulpymoney.libxulpymoney import OrderManager
from xulpymoney.libxulpymoneyfunctions import qmessagebox
from xulpymoney.ui.wdgOrdersAdd import wdgOrdersAdd
from xulpymoney.ui.wdgDisReinvest import wdgDisReinvest
from xulpymoney.ui.Ui_wdgOrders import Ui_wdgOrders
from xulpymoney.ui.frmInvestmentReport import frmInvestmentReport

class wdgOrders(QWidget, Ui_wdgOrders):
    def __init__(self, mem,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.orders=None 
         
        self.tblOrders.settings(self.mem, "wdgOrders")
        self.tblSellingPoints.settings(self.mem, "wdgOrders")
        self.mem.data.investments_active().myqtablewidget_sellingpoints(self.tblSellingPoints)
        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())
        self.wdgYear.initiate(self.orders.date_first_db_order().year,  datetime.date.today().year, datetime.date.today().year)
        
        
    @pyqtSlot()  
    def on_actionOrderNew_triggered(self):
        d=QDialog(self)     
        d.setModal(True)
        d.setWindowTitle(self.tr("Add new order"))
        w=wdgOrdersAdd(self.mem, None, None, d)
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_()    
        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())
    
    @pyqtSlot()  
    def on_actionOrderEdit_triggered(self):
        d=QDialog(self)     
        d.setModal(True)
        d.setWindowTitle(self.tr("Edit order"))
        w=wdgOrdersAdd(self.mem, self.orders.selected, self.orders.selected.investment, d)
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_()
        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())
        
    @pyqtSlot() 
    def on_actionOrderDelete_triggered(self):
        self.orders.remove(self.orders.selected)
        self.mem.con.commit()
        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())
        
        
    @pyqtSlot()
    def on_actionShowReinvest_triggered(self):
        if self.orders.selected.price==None or self.orders.selected.shares==None or self.orders.selected.investment.shares()==0:
            qmessagebox(self.tr("This order can't be simulated"))
            return
        
        
        
        d=QDialog()       
        d.resize(self.mem.settings.value("wdgOrders/qdialog_disreinvest", QSize(1024, 768)))
        d.setWindowTitle(self.tr("Order reinvest simulation"))
        w=wdgDisReinvest(self.mem, self.orders.selected.investment, False,  d)
        w.txtValorAccion.setText(self.orders.selected.price)
        w.txtSimulacion.setText(self.orders.selected.price*self.orders.selected.shares)
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_()
        self.mem.settings.setValue("frmInvestmentReport/qdialog_disreinvest", d.size())
                
    @pyqtSlot()
    def on_actionShowReinvestSameProduct_triggered(self):
        if self.orders.selected.price==None or self.orders.selected.shares==None or self.orders.selected.investment.shares()==0:
            qmessagebox(self.tr("This order can't be simulated"))
            return
        
        d=QDialog()       
        d.resize(self.mem.settings.value("wdgOrders/qdialog_disreinvest", QSize(1024, 768)))
        d.setWindowTitle(self.tr("Order reinvest simulation with all investments with the same product"))
        w=wdgDisReinvest(self.mem, self.orders.selected.investment, True,  d)
        w.txtValorAccion.setText(self.orders.selected.price)
        w.txtSimulacion.setText(self.orders.selected.price*self.orders.selected.shares)
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_()
        self.mem.settings.setValue("frmInvestmentReport/qdialog_disreinvest", d.size())
        
    @pyqtSlot() 
    def on_actionExecute_triggered(self):
        if self.orders.selected.investment.questionbox_inactive()==QMessageBox.No:#It's not active, after all.
            return
        
        if self.orders.selected.executed==None:#Only adds operation if it's not executed
            w=frmInvestmentReport(self.mem, self.orders.selected.investment, self)
            w.frmInvestmentOperationsAdd_initiated.connect(self.load_OrderData)
            w.on_actionOperationAdd_triggered()        
            w.exec_()
        
        if self.orders.selected.executed==None:
            self.orders.selected.executed=self.mem.localzone.now()#Set execution
        else:
            self.orders.selected.executed=None#Remove execution
        self.orders.selected.save()
        self.mem.con.commit()

        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())
        
    def load_OrderData(self, frm):
        """Carga los datos de la orden en el frmInvestmentOperationsAdd"""
        if self.orders.selected.shares<0:
            frm.cmbTiposOperaciones.setCurrentIndex(frm.cmbTiposOperaciones.findData(5))#Sale
        else:
            frm.cmbTiposOperaciones.setCurrentIndex(frm.cmbTiposOperaciones.findData(4))#Purchase
        frm.txtAcciones.setText(self.orders.selected.shares)
        frm.wdg2CPrice.setTextA(self.orders.selected.price)

    @pyqtSlot(int)     
    def on_cmbMode_currentIndexChanged(self, index):
        if index==0:#Current
            self.wdgYear.hide()            
            self.orders=OrderManager(self.mem).init__from_db("""
                SELECT * 
                FROM 
                    ORDERS
                WHERE
                    EXPIRATION>=NOW()::DATE AND
                    EXECUTED IS NULL
                ORDER BY DATE
           """)
            self.orders.order_by_percentage_from_current_price()
        elif index==1: #show expired
            self.wdgYear.show()
            self.orders=OrderManager(self.mem).init__from_db(self.mem.con.mogrify("""
                SELECT * 
                FROM 
                    ORDERS
                WHERE
                    DATE BETWEEN '%s-1-1' AND '%s-12-31' AND
                    EXPIRATION<NOW()::DATE AND
                    EXECUTED IS NULL
                ORDER BY DATE
           """, (self.wdgYear.year, self.wdgYear.year)))
            self.orders.order_by_expiration()
        elif index==2: #show executed
            self.wdgYear.show()
            self.orders=OrderManager(self.mem).init__from_db(self.mem.con.mogrify("""
                SELECT * 
                FROM 
                    ORDERS
                WHERE
                    DATE BETWEEN '%s-1-1' AND '%s-12-31' AND
                    EXECUTED IS NOT NULL
                ORDER BY DATE
           """, (self.wdgYear.year, self.wdgYear.year)))
            self.orders.order_by_execution()
        else:
            self.wdgYear.show()
            self.orders=OrderManager(self.mem).init__from_db(self.mem.con.mogrify("""
                SELECT * 
                FROM 
                    ORDERS
                WHERE
                    DATE BETWEEN '%s-1-1' AND '%s-12-31'
                ORDER BY DATE
           """, (self.wdgYear.year, self.wdgYear.year)))
            self.orders.order_by_date()
        self.orders.myqtablewidget(self.tblOrders)
       
    def on_tblOrders_customContextMenuRequested(self,  pos):
        if self.orders.selected==None:
            self.actionOrderDelete.setEnabled(False)
            self.actionOrderEdit.setEnabled(False)
            self.actionExecute.setEnabled(False)
            self.actionShowReinvest.setEnabled(False)
            self.actionShowReinvestSameProduct.setEnabled(False)
        else:
            self.actionOrderDelete.setEnabled(True)
            self.actionOrderEdit.setEnabled(True)
            self.actionExecute.setEnabled(True)
            if self.orders.selected.executed==None:
                self.actionExecute.setText("Execute order")
            else:
                self.actionExecute.setText("Remove execution time")
            self.actionShowReinvest.setEnabled(True)
            self.actionShowReinvestSameProduct.setEnabled(True)
                
                
            
        menu=QMenu()
        menu.addAction(self.actionOrderNew)
        menu.addSeparator()
        menu.addAction(self.actionOrderEdit)
        menu.addAction(self.actionOrderDelete)
        menu.addSeparator()
        menu.addAction(self.actionExecute)        
        menu.addSeparator()
        menu.addAction(self.actionShowReinvest)
        menu.addAction(self.actionShowReinvestSameProduct)
        menu.exec_(self.tblOrders.mapToGlobal(pos))

    def on_tblOrders_itemSelectionChanged(self):
        self.orders.selected=None
        for i in self.tblOrders.selectedItems():
            if i.column()==0:#only once per row
                self.orders.selected=self.orders.arr[i.row()]
                
    def on_wdgYear_changed(self):
        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())
        



import argparse
from PyQt5.QtWidgets import QApplication
from PyQt5.QtCore import QUrl,  QEventLoop
from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEngineProfile
from decimal import Decimal
import logging
import sys

import time
from xulpymoney.libxulpymoney import MemSources
from xulpymoney.libxulpymoneyfunctions import  addDebugSystem,  addCommonToArgParse








## NO FUNCIONA ##








class CurrentPriceTicker:
    def __init__(self,ticker, xulpymoney, stockmarket):
        self.ticker=ticker
        self.xulpymoney=xulpymoney
        self.stockmarket=stockmarket
        self.dtaware=self.stockmarket.estimated_datetime_for_intraday_quote()
        self.price=None

    def __repr__(self):
        if self.xulpymoney!=None:
            return "PRICE | XULPYMONEY | {} | {} | {}".format(self.xulpymoney, self.dtaware, self.price)
        else:
            return "PRICE | STOCKMARKET | XX | TICKER | {} | {} | {}".format(self.ticker, self.dtaware , self.price)

    def get_price(self):
        class Render(QWebEngineView):
            def __init__(self, url):
                QWebEngineView.__init__(self)
                self.loop=QEventLoop()#Queda en un loop hasta que acaba la carga de todas las páginas
                self.loadFinished.connect(self._loadFinished)
                self.pages=[]
                self.page().profile().cookieStore().deleteAllCookies()
                self.page().profile().setPersistentCookiesPolicy(QWebEngineProfile.NoPersistentCookies)
                self.load(QUrl(url))
                self.loop.exec()

            def numPages(self):
                return len(self.pages)

            def _loadFinished(self, result):
                """
                   This is an async call, you need to wait for this to be called before closing the app
                """
                self.page().toHtml(self._callable)

            def _callable(self, data):
                """
                   Se llama cada vez que una página es cargada con el html en data
                """                
                time.sleep(0.25)
                self.pages.append(data)
                self.loop.quit()#CUIDADO DEBE ESTAR EN EL ULTIMO
        ###########################
        url='https://es.finance.yahoo.com/quote/{0}?p={0}'.format(self.ticker)
        logging.debug("Searching in {}".format(url))

        r=Render(url)
        web=r.pages[0]
        if web==None:
            print ("ERROR | FETCHED EMPTY PAGE")
            sys.exit(0)
        try:
            ## ESTAN AHORA CON AVISO OAUTH
            if web.find("Mercado abierto")!=-1:
                
                logging.debug("Mercado abierto")
                self.price=web.split('"regularMarketPrice":{"raw":')[1]#Antes
                self.price=self.price.split(',"fmt":')[0].replace(".","")#Después
                logging.debug(self.price)
                self.price=self.price.replace(",", ".")
                self.price=Decimal(self.price)
                logging.debug(self.price)
            else: # Mercado cerrado
                logging.debug("Mercado cerrado")
                self.price=web.split('data-reactid="35">')[3]#Antes
                self.price=self.price.split('</span><span clas')[0].replace(".","")#Después
                self.price=self.price.replace(",", ".")
                logging.debug(self.price)
                self.price=Decimal(self.price)

        except:
            print ("ERROR | GET PRICE FAILED: PRECIO {}. DATETIME {}".format(self.price, self.dtaware))
            sys.exit(0)
    
def main():
    app = QApplication(sys.argv)
    parser=argparse.ArgumentParser()
    parser.add_argument('--TICKER_XULPYMONEY', help='XULPYMONEY code', nargs=2, metavar="VALUE", required=True)
    parser.add_argument('--STOCKMARKET', help='Stock market id', metavar="ID", required=True)
    addCommonToArgParse(parser)
    args=parser.parse_args()        
    addDebugSystem(args)

    mem=MemSources()
    stockmarket=mem.stockmarkets.find_by_id(int(args.STOCKMARKET))
    logging.info(stockmarket)
    if args.TICKER_XULPYMONEY:
        s=CurrentPriceTicker(args.TICKER_XULPYMONEY[0], args.TICKER_XULPYMONEY[1], stockmarket)
        s.get_price()
        print(s)




import datetime
from decimal import Decimal
from xulpymoney.libxulpymoney import Percentage, qmessagebox, qdate, qleft, qempty, qright,  Money
from xulpymoney.libxulpymoneyfunctions import relation_gains_risk
from xulpymoney.libmanagers import ObjectManager_With_IdDate
from xulpymoney.libxulpymoneytypes import eQColor, eInvestmentTypePosition
from PyQt5.QtCore import QObject
from PyQt5.QtWidgets import QTableWidgetItem

## Class that register a purchase opportunity
class Opportunity:
    ## Constructor with the following attributes combination
    ## 1. Opportunity(mem). Crete a Opportunity object with all attributes set to None
    ## 1. Opportunity(mem, row). Create an Opportunity from a db row, generated in a database query
    ## 2. Opportunity(mem, date, removed, executed, entry, target, stoploss, products_id, id). Create a Opportunity passing all attributes
    ## @param mem MemXulpymoney object
    ## @param row Dictionary of a database query cursor
    ## @param date datetime.date object with the date of the Opportunity
    ## @param removed datetime.date object when the Opportunity was removed
    ## @param executed datetime.date object when the Opportunity was executed
    ## @param entry decimal.Decimal object with the Opportunity entry
    ## @param products_id Integer with the product id, after the constructur this integer is converted to self.product using mem
    ## @param id Integer that sets the id of an Opportunity. If id=None it's not in the database. id is set in the save method
    def __init__(self, *args):
        def init__db_row( row):
            init__create(row['date'], row['removed'], row['executed'], row['entry'], row['target'], row['stoploss'], row['products_id'], row['short'], row['id'])
        def init__create(date, removed, executed, entry, target, stoploss, products_id, short, id):
            self.date=date
            self.removed=removed
            self.executed=executed
            self.entry=entry
            self.stoploss=stoploss
            self.target=target
            if products_id!=None:
                self.product=self.mem.data.products.find_by_id(products_id)
                self.product.needStatus(1)
            else:
                self.product=None
            self.short=short
            self.id=id
        self.mem=args[0]
        if len(args)==1:
            init__create(None, None, None, Decimal('0'), Decimal('0'), Decimal('0'), None, None)
        if len(args)==2:
            init__db_row(args[1])
        if len(args)==9:
            init__create(*args[1:])

    ## In Spanish is said "Está vigente"
    def is_in_force(self):
        if self.is_removed()==False and self.is_executed()==False:
            return True
        return False

    def is_removed(self):
        if self.removed!=None:
            return True
        return False
        
    def is_executed(self):
        if self.executed!=None:
            return True
        return False

    def save(self, autocommit=False):
        cur=self.mem.con.cursor()
        if self.id==None:#insertar
            cur.execute("insert into opportunities(date, removed, executed, entry, target, stoploss, short, products_id) values (%s, %s, %s, %s, %s, %s, %s, %s) returning id", 
            (self.date,  self.removed, self.executed, self.entry, self.target, self.stoploss, self.short, self.product.id))
            self.id=cur.fetchone()[0]
        else:
            cur.execute("update opportunities set date=%s, removed=%s, executed=%s, entry=%s, target=%s, stoploss=%s, products_id=%s, short=%s where id=%s", (self.date,  self.removed, self.executed, self.entry, self.target, self.stoploss, self.product.id, self.short, self.id))
        if autocommit==True:
            self.mem.con.commit()
        cur.close()
        
    def relation_gains_risk(self):
        return relation_gains_risk(self.target, self.entry, self.stoploss)
        
    def remove(self):
        cur=self.mem.con.cursor()
        cur.execute("delete from opportunities where id=%s", (self.id, ))
        cur.close()
        
    ##Calculates percentage from current price to order entry
    def percentage_from_current_price(self):
        return Percentage(self.entry-self.product.result.basic.last.quote, self.product.result.basic.last.quote)
        
    ## @param dInvested Decimal with the invested amount in product currency
    ## @return Money with the gains in opportunity product currrency
    def m_target_gains(self, dInvested):
        try:
            shares=dInvested/self.entry
            value= (self.target-self.entry)*shares
            value=-value if self.short==True else value
        except:
            value= Decimal(0)
        return Money(self.mem, value, self.product.currency)

    ## @param dInvested Decimal with the invested amount in product currency
    ## @return Money with the gains in opportunity product currrency
    def m_stop_loss_losses(self, dInvested):
        try:
            shares=dInvested/self.entry
            value= (self.stoploss-self.entry)*shares
            value=-value if self.short==True else value
        except:
            value= Decimal(0)
        return Money(self.mem, value, self.product.currency)

## Manage Opportunities
class OpportunityManager(ObjectManager_With_IdDate, QObject):
    def __init__(self, mem):
        QObject.__init__(self)
        ObjectManager_With_IdDate.__init__(self)
        self.mem=mem

    def init__from_db(self, sql):
        cur=self.mem.con.cursor()
        cur.execute(sql)
        for row in cur:
            self.append(Opportunity(self.mem, row))
        cur.close()
        return self

    ## Removes from array and from database. It doesn't make a database commit
    def remove(self, order):
        """Remove from array"""
        ObjectManager_With_IdDate.remove(self, order)#Remove from array
        order.remove()#Database

    def order_by_removed(self):
        self.arr=sorted(self.arr, key=lambda o:o.removed)

    def order_by_executed(self):
        self.arr=sorted(self.arr, key=lambda o:o.executed)

    def order_by_percentage_from_current_price(self):
        try:
            self.arr=sorted(self.arr, key=lambda o:o.percentage_from_current_price(), reverse=True)
        except:            
            qmessagebox(self.tr("I couldn't order data due to they have null values"))

    ## Returns a datetime.date object with the date of the first opportunity in the database
    def date_of_the_first_database_oppportunity(self):
        cur=self.mem.con.cursor()
        cur.execute("select date from orders order by date limit 1")
        r=cur.fetchone()
        cur.close()
        if r==None:#To avoid crashed returns today if null
            return datetime.date.today()
        else:
            return r[0]

    ## @param table
    ## @invested Decimal with the amount to invest in product currency
    def myqtablewidget(self, table, dInvested):
        table.setColumnCount(12)
        table.setHorizontalHeaderItem(0, QTableWidgetItem(self.tr("Date")))
        table.setHorizontalHeaderItem(1, QTableWidgetItem(self.tr("Removed")))
        table.setHorizontalHeaderItem(2, QTableWidgetItem(self.tr("Product")))
        table.setHorizontalHeaderItem(3, QTableWidgetItem(self.tr("Current price")))
        table.setHorizontalHeaderItem(4, QTableWidgetItem(self.tr("Entry")))
        table.setHorizontalHeaderItem(5, QTableWidgetItem(self.tr("Target")))
        table.setHorizontalHeaderItem(6, QTableWidgetItem(self.tr("Stop loss")))
        table.setHorizontalHeaderItem(7, QTableWidgetItem(self.tr("Gains/Risk")))
        table.setHorizontalHeaderItem(8, QTableWidgetItem(self.tr("% from current")))
        table.setHorizontalHeaderItem(9, QTableWidgetItem(self.tr("Executed")))
        table.setHorizontalHeaderItem(10, QTableWidgetItem(self.tr("Target gains")))
        table.setHorizontalHeaderItem(11, QTableWidgetItem(self.tr("Stop loss losses")))
        table.applySettings()
        table.clearContents()
        table.setRowCount(self.length())
        for i, p in enumerate(self.arr):
            table.setItem(i, 0, qdate(p.date))
            table.item(i, 0).setIcon(eInvestmentTypePosition.qicon_boolean(p.short))
            table.setItem(i, 1, qdate(p.removed))      
            table.setItem(i, 2, qleft(p.product.name))
            table.setItem(i, 3, p.product.result.basic.last.money().qtablewidgetitem())
            table.setItem(i, 4, p.product.currency.qtablewidgetitem(p.entry))
            table.setItem(i, 5, p.product.currency.qtablewidgetitem(p.target))
            table.setItem(i, 6, p.product.currency.qtablewidgetitem(p.stoploss))
            table.setItem(i, 7, qright(p.relation_gains_risk()))
            if p.is_in_force():
                table.setItem(i, 8, p.percentage_from_current_price().qtablewidgetitem())
            else:
                table.setItem(i, 8, qempty())
            if p.is_executed():
                table.setItem(i, 9, qdate(p.executed))
            else:
                table.setItem(i, 9, qempty())
                
            table.setItem(i, 10, p.m_target_gains(dInvested).qtablewidgetitem())
            table.setItem(i, 11, p.m_stop_loss_losses(dInvested).qtablewidgetitem())
                
            #Color
            if p.is_executed():
                for column in range (table.columnCount()):
                    table.item(i, column).setBackground(eQColor.Green)                     
            elif p.is_removed():
                for column in range (table.columnCount()):
                    table.item(i, column).setBackground(eQColor.Red)     
                    
            if p.is_executed()==False and p.is_removed()==False:#Color if current oportunity
                if p.percentage_from_current_price().value_100()>Decimal(0):
                    table.item(i, 3).setBackground(eQColor.Green)




## @namesapace xulpymoney.ui.frmMain
## @brief User interface main window.

from PyQt5.QtCore import pyqtSlot, QProcess, QUrl,  QSize
from PyQt5.QtGui import QIcon, QDesktopServices
from PyQt5.QtWidgets import QMainWindow,  QWidget, QLabel, QMessageBox, QProgressDialog, QDialog,  QApplication, QVBoxLayout
import os
from xulpymoney.ui.Ui_frmMain import Ui_frmMain
from xulpymoney.libxulpymoney import AssetsReport, Product, ProductManager
from xulpymoney.libxulpymoneyfunctions import list2string, qmessagebox, sync_data
from xulpymoney.libxulpymoneytypes import eProductType
from xulpymoney.version import __versiondate__
from xulpymoney.ui.frmAccess import frmAccess
from xulpymoney.ui.myqlineedit import myQLineEdit
from xulpymoney.ui.wdgTotal import wdgTotal
from xulpymoney.ui.wdgDividendsReport import wdgDividendsReport
from xulpymoney.ui.wdgInvestmentClasses import wdgInvestmentClasses
from xulpymoney.ui.wdgAPR import wdgAPR
from xulpymoney.ui.wdgAccounts import wdgAccounts
from xulpymoney.ui.wdgBanks import wdgBanks
from xulpymoney.ui.wdgConcepts import wdgConcepts
from xulpymoney.ui.wdgCalculator import wdgCalculator
from xulpymoney.ui.wdgCuriosities import wdgCuriosities
from xulpymoney.ui.wdgIndexRange import wdgIndexRange
from xulpymoney.ui.wdgInvestmentsOperations import wdgInvestmentsOperations
from xulpymoney.ui.wdgInvestmentsRanking import wdgInvestmentsRanking
from xulpymoney.ui.frmAuxiliarTables import frmAuxiliarTables
from xulpymoney.ui.frmTransfer import frmTransfer
from xulpymoney.ui.frmSettings import frmSettings
from xulpymoney.ui.wdgOrders import wdgOrders
from xulpymoney.ui.wdgOpportunities import wdgOpportunities
from xulpymoney.ui.wdgProducts import wdgProducts
from xulpymoney.ui.wdgProductsComparation import wdgProductsComparation
from xulpymoney.ui.wdgQuotesUpdate import wdgQuotesUpdate
from stdnum.isin import is_valid

from xulpymoney.ui.wdgLastCurrent import wdgLastCurrent

class frmMain(QMainWindow, Ui_frmMain):
    def __init__(self, mem, parent = 0,  flags = False):
        QMainWindow.__init__(self, None)
        self.setupUi(self)
        self.showMaximized()
        
        self.mem=mem
        self.mem.con.inactivity_timeout.connect(self.inactivity_timeout)        
        self.sqlvacio="select * from products where id=-999999"
        
        self.w=QWidget()       
        self.statusBar.addWidget(QLabel(self.mem.con.url_string()))

        self.mem.load_db_data() ##CARGA TODOS LOS DATOS Y LOS VINCULA       
  
        if self.mem.con.is_superuser():
            self.setWindowTitle(self.tr("Xulpymoney 2010-{0} \xa9 (Admin mode)").format(__versiondate__.year))#print ("Xulpymoney 2010-{0} © €".encode('unicode-escape'))
            self.setWindowIcon(self.mem.qicon_admin())
        else:
            self.setWindowTitle(self.tr("Xulpymoney 2010-{0} \xa9").format(__versiondate__.year))
            self.actionDocumentsPurge.setEnabled(False)

    ## Checks if products.xlsx version in Internet is older than db products.xlsx version in database
    def __checks_version_of_products_xlsx(self):
        from xulpymoney.github import get_file_modification_dt
        dbversion=self.mem.settingsdb.value("Version of products.xlsx",None)
        internetversion=get_file_modification_dt("turulomio","xulpymoney","products.xlsx")
        if self.mem.settingsdb.value("Version of products.xlsx", None)==None or dbversion<internetversion:
            self.actionProductsUpdate.setText(self.tr("Update products from Internet (NEEDED)"))
            self.actionRangeReport.setIcon(QIcon(":/xulpymoney/cloud_download_needed.png"))


    def actionsEnabled(self, bool):
        self.menuBar.setEnabled(bool)
        self.toolBar.setEnabled(bool)
        
    @pyqtSlot()
    def on_actionGlobalReport_triggered(self):
        file="AssetsReport.odt"
        doc=AssetsReport(self.mem, file)
        doc.generate()
        
        
        #Open file
        if os.path.exists("/usr/bin/lowriter"):
            QProcess.startDetached("lowriter", [file, ] )
        elif os.path.exists("/usr/bin/kfmclient"):
            QProcess.startDetached("kfmclient", ["openURL", file] )
        elif os.path.exists("/usr/bin/gnome-open"):
            QProcess.startDetached( "gnome-open '" + file + "'" );
        else:         
            QDesktopServices.openUrl(QUrl("file://"+file));


    def inactivity_timeout(self):
        self.hide()
        qmessagebox(self.tr("Disconnecting due to {} minutes of inactivity.".format(self.mem.con.inactivity_timeout_minutes)))
        self.on_actionExit_triggered()


    @pyqtSlot()  
    def on_actionExit_triggered(self):
        self.mem.__del__()
        print ("App correctly closed")
        self.close()
        self.destroy()
        
    @pyqtSlot()
    def on_actionAbout_triggered(self):
        from xulpymoney.ui.frmAbout import frmAbout
        fr=frmAbout(self.mem, self, "frmabout")
        fr.open()

    @pyqtSlot()  
    def on_actionBanks_triggered(self):
        self.w.close()
        self.w=wdgBanks(self.mem, self)
                
        self.layout.addWidget(self.w)
        self.w.show()
        
    @pyqtSlot()  
    def on_actionCalculator_triggered(self):
        d=QDialog(self)        
        d.setFixedSize(850, 850)
        d.setWindowTitle(self.tr("Investment calculator"))
        w=wdgCalculator(self.mem, self)
        w.setProduct(self.mem.data.products.find_by_id(int(self.mem.settings.value("wdgCalculator/product", "0"))))
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        if w.hasProducts==True:
            d.show()
        else:
            d.close()
        
    @pyqtSlot()  
    def on_actionConcepts_triggered(self):
        self.w.close()
        self.w=wdgConcepts(self.mem, self)
                
        self.layout.addWidget(self.w)
        self.w.show()
        
    @pyqtSlot()  
    def on_actionAccounts_triggered(self):
        self.w.close()
        self.w=wdgAccounts(self.mem, self)
                
        self.layout.addWidget(self.w)
        self.w.show()
    
    @pyqtSlot()  
    def on_actionMemory_triggered(self):        
        self.mem.data.load()
        
        
    @pyqtSlot()  
    def on_actionDividendsReport_triggered(self):
        self.w.close()
        self.w=wdgDividendsReport(self.mem, self)
                
        self.layout.addWidget(self.w)
        self.w.show()
        
    @pyqtSlot()  
    def on_actionInvestmentsClasses_triggered(self):
        self.w.close()
        self.w=wdgInvestmentClasses(self.mem, self)
        self.layout.addWidget(self.w)
        self.w.update()
        self.w.show()

    @pyqtSlot()  
    def on_actionTotalReport_triggered(self):
        self.w.close()
        self.w=wdgTotal(self.mem, self)
                
        self.layout.addWidget(self.w)
        self.w.show()

    @pyqtSlot()  
    def on_actionEvolutionReport_triggered(self):
        self.w.close()
        self.w=wdgAPR(self.mem, self)
              
        self.layout.addWidget(self.w)
        self.w.show()
        
    @pyqtSlot()  
    def on_actionHelp_triggered(self):
        try: ## Remove when qwebwenginewidgets work again
            from xulpymoney.ui.frmHelp import frmHelp
            w=frmHelp(self.mem, self)
            w.show()
        except:
            QDesktopServices.openUrl(QUrl(self.mem.url_wiki))

    @pyqtSlot()  
    def on_actionIndexRange_triggered(self):
        if self.mem.data.benchmark.result.ohclDaily.length()==0:
            qmessagebox(self.tr("There isn't any benchmark data yet."))
            return
        
        self.w.close()
        self.w=wdgIndexRange(self.mem, self)
                
        self.layout.addWidget(self.w)
        self.w.show()


    @pyqtSlot()  
    def on_actionInvestments_triggered(self):
        from xulpymoney.ui.wdgInvestments import wdgInvestments
        self.w.close()
        self.w=wdgInvestments(self.mem, self)               
        self.layout.addWidget(self.w)
        self.w.show()

    @pyqtSlot()
    def on_actionInvestmentsZeroRisk_triggered(self):
        from xulpymoney.ui.wdgInvestments import wdgInvestmentsZeroRisk
        self.w.close()
        self.w=wdgInvestmentsZeroRisk(self.mem, self)
        self.layout.addWidget(self.w)
        self.w.show()

    @pyqtSlot()  
    def on_actionLastOperation_triggered(self):
        self.w.close()
        self.w=wdgLastCurrent(self.mem, self)               
        self.layout.addWidget(self.w)
        self.w.show()

    @pyqtSlot()  
    def on_actionInvestmentsOperations_triggered(self):
        self.w.close()
        self.w=wdgInvestmentsOperations(self.mem, self)
        self.layout.addWidget(self.w)
        self.w.show()

    @pyqtSlot()  
    def on_actionAuxiliarTables_triggered(self):
        w=frmAuxiliarTables(self.mem, self)
        w.exec_()
        
    @pyqtSlot()  
    def on_actionSettings_triggered(self):
        w=frmSettings(self.mem, self)
        w.exec_()
        self.retranslateUi(self)

    @pyqtSlot()  
    def on_actionTransfer_triggered(self):
        w=frmTransfer(self.mem, parent=self)
        w.exec_()
        self.on_actionAccounts_triggered()

    @pyqtSlot()  
    def on_actionCAC40_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.agrupations.dbstring().find("|CAC|")!=-1 and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()
        
    @pyqtSlot()  
    def on_actionActive_triggered(self):
        self.w.close()
        self.w=wdgProducts(self.mem,  "select * from products where obsolete=false order by name")

        self.layout.addWidget(self.w)
        self.w.show()
    
    @pyqtSlot()  
    def on_actionCuriosities_triggered(self):
        self.w.close()
        self.w=wdgCuriosities(self.mem,  self)
        self.layout.addWidget(self.w)
        self.w.show()
    
    @pyqtSlot()  
    def on_actionCurrenciesAll_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.Currency and p.obsolete==False:
                arrInt.append(p.id)
            
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()

    @pyqtSlot()  
    def on_actionDividends_triggered(self):
        """Shows products with current year estimations_dps and with quotes in current year"""
        self.w.close()
        prod=ProductManager(self.mem)
        prod.load_from_db("select * from products where id in (select id from estimations_dps where year=date_part('year',now()) and estimation is not null) and id in (select distinct(id) from quotes where date_part('year', datetime)=date_part('year',now()))")
        self.w=wdgProducts(self.mem,  prod.array_of_ids())
        self.layout.addWidget(self.w)
        self.w.on_actionSortDividend_triggered()
        self.w.show()

    @pyqtSlot()  
    def on_actionInvestmentRanking_triggered(self):
        self.w.close()
        self.w=wdgInvestmentsRanking(self.mem, self)
        self.layout.addWidget(self.w)
        self.w.show()
        
    @pyqtSlot()  
    def on_actionShowBenchmark_triggered(self):
        from xulpymoney.ui.frmProductReport import frmProductReport
        self.w.close()
        self.w=frmProductReport(self.mem, self.mem.data.benchmark)
        self.layout.addWidget(self.w)
        self.w.show()
    
    @pyqtSlot()  
    def on_actionSimulations_triggered(self):
        from xulpymoney.ui.wdgSimulations import wdgSimulations
        d=QDialog(self)
        d.resize(self.mem.settings.value("wdgSimulations/qdialog", QSize(1024, 768)))
        d.setWindowTitle(self.tr("Xulpymoney Simulations"))
        w=wdgSimulations(self.mem, d)
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_() 
        self.mem.settings.setValue("wdgSimulations/qdialog", d.size())

    @pyqtSlot()  
    def on_actionSyncProducts_triggered(self):
        self.w.hide()
        
        source=frmAccess(self.mem,  self)
        source.showLanguage(False)
        source.setLabel(self.tr("Please login to the source xulpymoney database"))
        source.txtPort.setText(self.mem.settings.value("frmMain/syncproducts_port", "5432"))
        source.txtServer.setText(self.mem.settings.value("frmMain/syncproducts_server", "127.0.0.1"))
        source.txtUser.setText(self.mem.settings.value("frmMain/syncproducts_user", "postgres"))
        source.txtDB.setText(self.mem.settings.value("frmMain/syncproducts_db", ""))
        source.txtPass.setFocus()
        source.exec_()
        if source.result()==QDialog.Rejected:             
            qmessagebox(self.tr("Error conecting to {} database in {} server").format(source.con.db, source.con.server))
            return
        else:
            if source.con.db.strip()==self.mem.con.db.strip() and source.con.server.strip()==self.mem.con.server.strip() and source.con.port==self.mem.con.port:
                m=QMessageBox()
                m.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
                m.setIcon(QMessageBox.Information)
                m.setText(self.tr("Databases can't be the same"))
                m.exec_()   
                return

            self.mem.settings.setValue("frmMain/syncproducts_port", source.txtPort.text())
            self.mem.settings.setValue("frmMain/syncproducts_server", source.txtServer.text())
            self.mem.settings.setValue("frmMain/syncproducts_user", source.txtUser.text())
            self.mem.settings.setValue("frmMain/syncproducts_db", source.txtDB.text())
                
            pd= QProgressDialog(QApplication.translate("Core","Syncing databases from {} ({}) to {} ({})").format(source.txtServer.text(), source.txtDB.text(), self.mem.con.server, self.mem.con.db), None, 0, 10)
            pd.setModal(True)
            pd.setWindowTitle(QApplication.translate("Core","Processing products..."))
            pd.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
            pd.forceShow()
            
            sync_data(source.con, self.mem.con, pd)
            
            self.mem.data.load()

    @pyqtSlot()  
    def on_actionNasdaq100_triggered(self):
        self.w.close()

        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.agrupations.dbstring().find("|NASDAQ100|")!=-1 and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()
            
    @pyqtSlot()  
    def on_actionISINDuplicado_triggered(self):
        self.w.close()
        cur=self.mem.con.cursor()
        
        
        #ßaca los isin duplicados buscando distintct isin, bolsa con mas de dos registros
        cur.execute("select isin, stockmarkets_id, count(*) as num from products  where isin!='' group by isin, stockmarkets_id having count(*)>1 order by num desc;")
        isins=set([])
        for row in cur:
            isins.add(row['isin'] )
        arrInt=[]
        if len(isins)>0:
            for p in self.mem.data.products.arr:
                if p.isin in isins:
                    arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()
        
    @pyqtSlot()  
    def on_actionMC_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.agrupations.dbstring().find("|MERCADOCONTINUO|")!=-1 and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()
        

    @pyqtSlot()  
    def on_actionETFAll_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.ETF and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()
        
    @pyqtSlot()  
    def on_actionETFObsolete_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.ETF and p.obsolete==True:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()
        
    @pyqtSlot()  
    def on_actionEurostoxx50_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.agrupations.dbstring().find("|EUROSTOXX|")!=-1 and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()
        

    @pyqtSlot()  
    def on_actionFavorites_triggered(self):
        if len(self.mem.favorites)==0:
            m=QMessageBox()
            m.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
            m.setIcon(QMessageBox.Information)
            m.setText(self.tr("There aren't favorite products"))
            m.exec_()     
            return
        self.w.close()
        self.w=wdgProducts(self.mem, self.mem.favorites)
        self.layout.addWidget(self.w)
        self.w.show()

    @pyqtSlot()  
    def on_actionSharesAll_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.Share and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()         
        
    @pyqtSlot()  
    def on_actionSharesObsolete_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.Share and p.obsolete==True:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()        
        
    @pyqtSlot()  
    def on_actionWarrantsAll_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.Warrant and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()          
        
    @pyqtSlot()  
    def on_actionWarrantsObsolete_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.Warrant and p.obsolete==True:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()          
        
    @pyqtSlot()  
    def on_actionWarrantsCall_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.Warrant and p.mode.id=='c' and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()              
    @pyqtSlot()  
    def on_actionWarrantsPut_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.Warrant and p.mode.id=='p' and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()              
        
    @pyqtSlot()  
    def on_actionWarrantsInline_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.Warrant and p.mode.id=='i' and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()      

    @pyqtSlot()  
    def on_actionFundsAll_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.Fund and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()        
        
    @pyqtSlot()  
    def on_actionFundsObsolete_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.Fund and p.obsolete==True:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()                        

    @pyqtSlot()  
    def on_actionBondsPublic_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.PublicBond and p.obsolete==False:
                arrInt.append(p.id)
            
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()                        

    @pyqtSlot()  
    def on_actionBondsPrivate_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.PrivateBond and p.obsolete==False:
                arrInt.append(p.id)
            
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()

    @pyqtSlot()  
    def on_actionPurchaseOpportunities_triggered(self):
        self.w.close()
        self.w=wdgOpportunities(self.mem, self)
        self.layout.addWidget(self.w)
        
    @pyqtSlot()  
    def on_actionPurgeAll_triggered(self):
        """Purga todas las quotes de todas inversión. """
        products=[]
        curms=self.mem.con.cursor()
        curms.execute("select * from products where id in ( select distinct( id) from quotes) order by name;")
        for row in curms:
            products.append(Product(self.mem).init__db_row(row))
        curms.close()
               
        
        pd= QProgressDialog(QApplication.translate("Core","Purging innecesary data from all products"), QApplication.translate("Core","Cancel"), 0,len(products))
        pd.setModal(True)
        pd.setWindowTitle(QApplication.translate("Core","Purging quotes from all products"))
        pd.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
        pd.setMinimumDuration(0)          
        counter=0      
        
        for i, inv in enumerate(products):
            pd.setValue(i)
            pd.setLabelText(QApplication.translate("Core","Purging quotes from {0}.\nTotal purged in global process: {1}").format(inv.name,  counter))
            pd.update()
            QApplication.processEvents()
            if pd.wasCanceled():
                self.mem.con.rollback()
                return
            pd.update()
            QApplication.processEvents()
            pd.update()            
            inv.result.all.load_from_db(inv)
            invcounter=inv.result.all.purge(progress=True)
            if invcounter==None:#Pulsado cancelar
                self.mem.con.rollback()
                break
            else:
                counter=counter+invcounter
                self.mem.con.commit()
        
        m=QMessageBox()
        m.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
        m.setIcon(QMessageBox.Information)
        m.setText(self.tr("{0} quotes have been purged from {1} products".format(counter, len(products))))
        m.exec_()    
        
    @pyqtSlot()  
    def on_actionBondsAll_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id in (eProductType.PrivateBond,  eProductType.PublicBond) and p.obsolete==False:
                arrInt.append(p.id)
            
        self.w=wdgProducts(self.mem,  arrInt)

        self.layout.addWidget(self.w)
        self.w.show()
        
    @pyqtSlot()  
    def on_actionBondsObsolete_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id in (eProductType.PrivateBond,  eProductType.PublicBond) and p.obsolete==True:
                arrInt.append(p.id)
            
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()

    @pyqtSlot()  
    def on_actionIbex35_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.agrupations.dbstring().find("|IBEX|")!=-1 and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()        

    @pyqtSlot()  
    def on_actionIndexesAll_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.Index:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()      
        
    @pyqtSlot()  
    def on_actionIndexesObsolete_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.type.id==eProductType.Index and p.obsolete==True:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()     
                
    @pyqtSlot()  
    def on_actionSP500_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.agrupations.dbstring().find("|SP500|")!=-1 and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()
    
    @pyqtSlot()  
    def on_actionProductsComparation_triggered(self):
        self.w.close()
        self.w=wdgProductsComparation(self.mem)

        self.layout.addWidget(self.w)
        self.w.show()            

    @pyqtSlot()  
    def on_actionProductsInvestmentActive_triggered(self):
        self.w.close()       
        prod=ProductManager(self.mem)
        prod.load_from_db("select * from products where id in (select products_id from inversiones where active=true) order by name")
        arrInt=[]
        for p in prod.arr:
            arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()        

    @pyqtSlot()  
    def on_actionProductsInvalidISIN_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.isin!=None and is_valid(p.isin)==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()

    @pyqtSlot()  
    def on_actionProductsWithoutQuotes_triggered(self):
        self.w.close()
        prod=ProductManager(self.mem)
        prod.load_from_db("select p.*,q.* from products p, quote(p.id, now()) q where p.id=q.id and q.quote is null and obsolete=False order by name")
        arrInt=[]
        for p in prod.arr:
            arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()       

    @pyqtSlot()  
    def on_actionProductsWithPriceVariation_triggered(self):
        self.w.close()        
        d=QDialog()       
        d.setWindowTitle(self.tr("Price variation"))
        lblDays=QLabel("Days")
        txtDays=myQLineEdit(d)
        txtDays.setText(90)
        lblVariation=QLabel("Variation")
        txtVariation=myQLineEdit(d)
        txtVariation.setText(-10)
        lay = QVBoxLayout(d)
        lay.addWidget(lblDays)
        lay.addWidget(txtDays)
        lay.addWidget(lblVariation)
        lay.addWidget(txtVariation)
        d.exec_()
        sql= "select * from products where is_price_variation_in_time(id, {}, now()::timestamptz-interval '{} day')=true and obsolete=False order by name".format(txtVariation.text(), txtDays.text())
        print(sql)
        self.w=wdgProducts(self.mem, sql)
        self.layout.addWidget(self.w)
        self.w.show()       

    @pyqtSlot()  
    def on_actionProductsWithOldPrice_triggered(self):
        self.w.close()
        prod=ProductManager(self.mem)
        prod.load_from_db("select p.* from products p, quote(p.id, now()) q where p.id=q.id and q.datetime<now() -interval '30 day' and obsolete=False order by name")
        arrInt=[]
        for p in prod.arr:
            arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()    
        
    @pyqtSlot()  
    def on_actionProductsInvestmentInactive_triggered(self):
        self.w.close()
        prod=ProductManager(self.mem)
        prod.load_from_db("select * from products where id in (select products_id from inversiones where active=false) order by name")
        arrInt=[]
        for p in prod.arr:
            arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()

    @pyqtSlot()  
    def on_actionProductsObsolete_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.obsolete==True:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()
                
    @pyqtSlot()  
    def on_actionProductsAutoUpdate_triggered(self):
        self.w.close()
        prod=ProductManager(self.mem)
        prod.load_from_db("""select * from products 
                where id in ({})
                order by name
                """.format(list2string(list(self.mem.autoupdate))))
        arrInt=[]
        for p in prod.arr:
            arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()       
    
    @pyqtSlot()  
    def on_actionProductsNotAutoUpdate_triggered(self):
        self.w.close()
        prod=ProductManager(self.mem)
        prod.load_from_db("""select * from products 
                where obsolete=false and id not in ({})
                order by name
                """.format(list2string(list(self.mem.autoupdate))))
        arrInt=[]
        for p in prod.arr:
            arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()            
        
    @pyqtSlot()  
    def on_actionProductsUser_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.id<0:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()
        
    @pyqtSlot()  
    def on_actionProductsUpdate_triggered(self):
        p=ProductManager(self.mem)
        p.update_from_internet()

        
    @pyqtSlot()  
    def on_actionProductsWithoutISIN_triggered(self):
        self.w.close()
        prod=ProductManager(self.mem)
        prod.load_from_db("select * from products  where obsolete=false and (isin is null or isin ='') order by name,id")
        arrInt=[]
        for p in prod.arr:
            arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()

    @pyqtSlot()  
    def on_actionTablasAuxiliares_triggered(self):
        w=frmAuxiliarTables(self.mem, self)
        w.tblTipos_reload()
        w.exec_()

                
    @pyqtSlot()  
    def on_actionXetra_triggered(self):
        self.w.close()
        arrInt=[]
        for p in self.mem.data.products.arr:
            if p.agrupations.dbstring().find("|DAX|")!=-1 and p.obsolete==False:
                arrInt.append(p.id)
        self.w=wdgProducts(self.mem,  arrInt)
        self.layout.addWidget(self.w)
        self.w.show()        
        
    @pyqtSlot()  
    def on_actionSearch_triggered(self):
        self.w.close()
        self.w=wdgProducts(self.mem)
        self.layout.addWidget(self.w)
        self.w.show()
        
    @pyqtSlot()  
    def on_actionOrders_triggered(self):  
        self.w.close()
        self.w=wdgOrders(self.mem, self)
        self.layout.addWidget(self.w)


    @pyqtSlot()  
    def on_actionPriceUpdates_triggered(self):  
        self.w.close()
        self.w=wdgQuotesUpdate(self.mem, self)
        self.layout.addWidget(self.w)



from PyQt5.QtCore import pyqtSlot, Qt
from PyQt5.QtWidgets import QWidget, QDialog, QVBoxLayout
from xulpymoney.ui.Ui_wdgCalculator import Ui_wdgCalculator
from xulpymoney.libxulpymoney import Percentage, Money
from xulpymoney.libxulpymoneytypes import eProductType
from xulpymoney.ui.wdgOrdersAdd import wdgOrdersAdd
from xulpymoney.ui.wdgProductHistoricalChart import wdgProductHistoricalBuyChart
from decimal import Decimal

class wdgCalculator(QWidget, Ui_wdgCalculator):
    def __init__(self, mem,  parent=None):
        """Compulsory fields are final price and amount"""
        QWidget.__init__(self, parent)
        self.setupUi(self)
        
        self.mem=mem

        self.table.settings(self.mem, "wdgCalculator")
        self.hasProducts=True#Permits to show/hide the widget from external dialog

        self.investments=None#SetINvestments of the selected product
        self.product=self.mem.data.products.find_by_id(int(self.mem.settings.value("wdgCalculator/product", -9999)))
        self.mem.data.investments.ProductManager_with_investments_distinct_products().qcombobox_not_obsolete(self.cmbProducts, self.product)
        self.lblAmount.setText(self.tr("Amount to invest in {} ({})").format(self.mem.localcurrency.id, self.mem.localcurrency.symbol))

    def setProduct(self,  product):
        self.cmbProducts.setCurrentIndex(self.cmbProducts.findData(product.id))
        self.txtAmount.setText(Decimal(self.mem.settingsdb.value("wdgIndexRange/invertir", "10000")))
        
    ## Sets an investment programatically
    def setInvestment(self, investment):
        self.cmbInvestments.setCurrentIndex(self.cmbInvestments.findData(investment.id))
        self.txtAmount.setText(Decimal(self.mem.settingsdb.value("wdgIndexRange/invertir", "10000")))

    def cmbPrice_load(self):       
        if self.product:
            self.cmbPrice.clear() 
            self.cmbPrice.addItem(self.tr("Penultimate price at {} in {} ({})".format(str(self.product.result.basic.penultimate.datetime)[:16], self.product.currency.id, self.product.currency.symbol)))
            self.cmbPrice.addItem(self.tr("Last price at {} in {} ({})".format(str(self.product.result.basic.last.datetime)[:16], self.product.currency.id, self.product.currency.symbol)))
            self.cmbPrice.setCurrentIndex(1)#Last price
        else:
            self.cmbPrice.clear()
        

    @pyqtSlot(int)  
    def on_cmbPrice_currentIndexChanged(self, index):
        """To invoke this function you must call self.cmbPrice.setCurrentIndex()"""
        if index==1:
            self.txtProductPrice.setText(self.product.result.basic.last.quote)
        else:
            self.txtProductPrice.setText(self.product.result.basic.penultimate.quote)
            
        self.txtFinalPrice.textChanged.disconnect()
        self.txtFinalPrice.setText(round(self.txtProductPrice.decimal()*Decimal(1+Decimal(self.spnProductPriceVariation.value())*self.txtLeveraged.decimal()/100), 6))
        self.txtFinalPrice.textChanged.connect(self.on_txtFinalPrice_textChanged)
        
        self.calculate()
            
    @pyqtSlot(int)  
    def on_cmbProducts_currentIndexChanged(self, index):
        """To invoke this function you must call self.cmbProducts.setCurrentIndex()"""
        self.product=self.mem.data.products.find_by_id(self.cmbProducts.itemData(index))
        if self.product!=None:
            self.lblFinalPrice.setText(self.tr("Final price in {} ({})").format(self.product.currency.id, self.product.currency.symbol))
            if self.product.hasSameLocalCurrency():
                self.lblShares.setText(self.tr("Shares calculated"))
            else:
                self.lblShares.setText(self.tr("Shares calculated with {} at {}").format(Money(self.mem, 1, self.mem.localcurrency).conversionFactorString(self.product.currency, self.mem.localzone.now()), self.mem.localzone.now()))
            
            #Fills self.cmbInvestments with all product investments or with zero shares product investments
            if self.chkWithoutShares.checkState()==Qt.Checked:
                self.investments=self.mem.data.investments.InvestmentManager_with_investments_with_the_same_product_with_zero_shares(self.product)
            else:
                self.investments=self.mem.data.investments.InvestmentManager_with_investments_with_the_same_product(self.product)
            self.investments.qcombobox(self.cmbInvestments, tipo=3, selected=None, obsolete_product=False, investments_active=None, accounts_active=None)
 
            self.mem.settings.setValue("wdgCalculator/product", self.product.id)
            
            self.cmbPrice_load()        
            self.txtLeveraged.setText(self.product.leveraged.multiplier)
            self.txtFinalPrice.textChanged.disconnect()
            self.txtFinalPrice.setText(round(self.txtProductPrice.decimal()*Decimal(1+Decimal(self.spnProductPriceVariation.value())*self.txtLeveraged.decimal()/100), 6))
            self.txtFinalPrice.textChanged.connect(self.on_txtFinalPrice_textChanged)
        
            self.calculate()

    @pyqtSlot(int)  
    def on_cmbInvestments_currentIndexChanged(self, index):
        """To invoke this function you must call self.cmbInvestments.setCurrentIndex()"""
        if index>=0:#Only enabled if some investment is selected
            self.cmdOrder.setEnabled(True)
        else:
            self.cmdOrder.setEnabled(False)
        self.investments.selected=self.mem.data.investments.find_by_id(self.cmbInvestments.itemData(index))


    ## This check filters investments showing ones with 0 currrent shares
    def on_chkWithoutShares_stateChanged(self, state):
        self.on_cmbProducts_currentIndexChanged(self.cmbProducts.currentIndex())
        
    @pyqtSlot()
    def on_cmdOrder_released(self):
        d=QDialog(self)     
        d.setModal(True)
        d.setWindowTitle(self.tr("Add new order"))
        w=wdgOrdersAdd(self.mem, None, self.investments.selected, d)
        w.txtShares.setText(self.txtShares.decimal())
        w.txtPrice.setText(self.txtFinalPrice.decimal())
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_()

    @pyqtSlot()
    def on_cmdGraph_released(self):
        self.product.needStatus(2)
        d=QDialog(self)     
        d.showMaximized()
        d.setWindowTitle(self.tr("Purchase graph"))
        lay = QVBoxLayout(d)
        wc=wdgProductHistoricalBuyChart()
        wc.setProduct(self.product, None)
        wc.setPrice(self.txtFinalPrice.decimal())
        wc.generate()
        wc.display()
        lay.addWidget(wc)
        d.exec_()

    @pyqtSlot(float)
    def on_spnProductPriceVariation_valueChanged(self, value):
        self.on_cmbProducts_currentIndexChanged(self.cmbProducts.currentIndex())
        
    def on_txtFinalPrice_textChanged(self):
        self.spnProductPriceVariation.valueChanged.disconnect()
        self.spnProductPriceVariation.setValue(100*(self.txtFinalPrice.decimal()-self.txtProductPrice.decimal())/self.txtProductPrice.decimal()/self.txtLeveraged.decimal())
        self.spnProductPriceVariation.valueChanged.connect(self.on_spnProductPriceVariation_valueChanged)
        self.calculate()

    def on_txtAmount_textChanged(self):
        self.calculate()

    def calculate(self):
        """Checks if compulsory fields are ok, if not changes style to red, else calculate table and shares"""
        if self.txtAmount.isValid() and self.txtFinalPrice.isValid():
            finalprice=Money(self.mem, self.txtFinalPrice.decimal(), self.product.currency).convert(self.mem.localcurrency).amount
            if self.product.type.id in (eProductType.Share, eProductType.ETF):#Shares
                self.txtShares.setText(round(self.txtAmount.decimal()/finalprice, 0))
            else:
                self.txtShares.setText(round(self.txtAmount.decimal()/finalprice, 6))
            porcentages=[2.5, 5, 7.5, 10, 15, 30]
            self.table.clearContents()
            self.table.setRowCount(len(porcentages))
            for i, tpc in enumerate(porcentages):        
                self.table.setItem(i, 0, Percentage(tpc, 100).qtablewidgetitem())
                tpcprice= self.txtFinalPrice.decimal()*Decimal(1+tpc/100)
                self.table.setItem(i, 1, self.product.currency.qtablewidgetitem(tpcprice))       
                self.table.setItem(i, 2, self.product.currency.qtablewidgetitem(self.txtShares.decimal()*(tpcprice-self.txtFinalPrice.decimal())))



import datetime
import logging
import pytz
from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QWidget, QDialog, QVBoxLayout
from xulpymoney.ui.Ui_wdgDisReinvest import Ui_wdgDisReinvest
from xulpymoney.libxulpymoney import InvestmentOperation, Money, Percentage,  Quote
from xulpymoney.libxulpymoneyfunctions import qmessagebox
from xulpymoney.ui.wdgOrdersAdd import wdgOrdersAdd
from decimal import Decimal
from xulpymoney.ui.wdgProductHistoricalChart import wdgProductHistoricalReinvestChart

class wdgDisReinvest(QWidget, Ui_wdgDisReinvest):
    def __init__(self, mem, inversion,  allProduct=False,  parent=None):
        """
            Parameter allProduct true, make study with all active investments  of the product
        """
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem            

        if allProduct==False:
            self.investment=inversion
        else:
            self.investment=self.mem.data.investments_active().investment_merging_current_operations_with_same_product(inversion.product)
            
                
        self.txtValorAccion.setText(self.investment.product.result.basic.last.quote)
        self.txtSimulacion.setText(Decimal(self.mem.settingsdb.value("wdgIndexRange/invertir", "10000")))
        self.tabOpAcHi.setCurrentIndex(1)
        
        self.tblGainsAfter.settings(self.mem, "wdgDisReinvest")
        self.tblGainsBefore.settings(self.mem, "wdgDisReinvest")
        self.tblInvestmentsActualDespues.settings(self.mem, "wdgDisReinvest")
        self.tblInvestmentsActualDespuesAt.settings(self.mem, "wdgDisReinvest")
        self.tblInvestmentsActualAntesAt.settings(self.mem, "wdgDisReinvest")
        self.tblInvestmentsActualAntes.settings(self.mem, "wdgDisReinvest")
        self.tblInvestmentsHistoricas.settings(self.mem, "wdgDisReinvest")
        self.tblOperaciones.settings(self.mem, "wdgDisReinvest")
        
        if self.investment.op_actual.length()==0:
            qmessagebox(self.tr("There aren't shares for this investment"))
            return
            
        self.investment.op_actual.myqtablewidget(self.tblInvestmentsActualAntes)
        self.on_radRe_clicked()

    def shares(self):
        resultado=Decimal(0)
        
        if self.radDes.isChecked():#DESINVERSION
            perdida=Money(self.mem, self.txtSimulacion.decimal(),self.investment.product.currency)#Va disminuyendo con las distintas operaciones
            q=Quote(self.mem).init__create(self.investment.product, datetime.datetime.now(pytz.timezone(self.mem.localzone.name)), self.txtValorAccion.decimal())
            for rec in self.investment.op_actual.arr:
                pendiente=rec.pendiente(q)
                if (perdida+pendiente).isZero():
                    resultado=resultado+rec.shares
                    break
                elif (perdida+pendiente).isGTZero():
                    resultado=resultado+rec.shares
                    perdida=perdida+pendiente
                elif (perdida+pendiente).isLTZero():
                    # Si de tantas acciones queda pendiente "pendiente"
                    # X                                queda la perdida
                    acciones=abs(int(perdida.amount*rec.shares/pendiente.amount))
                    resultado=resultado+Decimal(acciones)#Se resta porque se debe calcular antes de quitarse el pendiente
                    break
        else:#REINVERSION
            resultado=Decimal(int(self.txtSimulacion.decimal()/self.txtValorAccion.decimal()))
        return resultado
            
    @pyqtSlot() 
    def on_radDes_clicked(self):
        self.lblTitulo.setText(self.tr("Divest simulation of {0}").format(self.investment.name))
        self.lblSimulacion.setText(self.tr("Divest loss to asume"))
        self.lblValor.setText(self.tr("Selling price (Current: {})").format(self.investment.product.currency.string(self.investment.product.result.basic.last.quote)))
        self.tabAB.setCurrentIndex(3)
        self.cmdOrder.setEnabled(False)
        
    @pyqtSlot() 
    def on_radRe_clicked(self):
        self.lblTitulo.setText(self.tr("Reinvest simulation of {0}").format(self.investment.name))
        self.lblSimulacion.setText(self.tr("Amount to reinvest"))
        self.lblValor.setText(self.tr("Purchase price (Current: {})").format(self.investment.product.currency.string(self.investment.product.result.basic.last.quote)))
        self.tabAB.setCurrentIndex(3)
        self.cmdOrder.setEnabled(False)
   
    def on_cmd_released(self): 
        self.sim_op=None
        self.sim_opactual=None
        self.sim_ophistorica=None
        
        
        at=Quote(self.mem).init__create(self.investment.product, datetime.datetime.now(), self.txtValorAccion.decimal())
        
        self.tabAB.setTabText(0, self.tr("After at {}").format(self.investment.product.currency.string(self.investment.product.result.basic.last.quote)))
        self.tabAB.setTabText(1, self.tr("After at {}").format(self.investment.product.currency.string(at.quote)))
        self.tabAB.setTabText(2, self.tr("Before at {}").format(self.investment.product.currency.string(at.quote)))
        self.tabAB.setTabText(3, self.tr("Before at {}").format(self.investment.product.currency.string(self.investment.product.result.basic.last.quote)))

        if self.txtSimulacion.decimal()<=Decimal('0'):
            qmessagebox(self.tr("Simulation value must be positive"))
            return
            
        valor_accion=self.txtValorAccion.decimal()
        impuestos=0
        comision=self.txtComision.decimal()
        
        if valor_accion==0:
            qmessagebox(self.tr("Share price can't be 0"))
            return
        
        acciones=self.shares()
        importe=valor_accion*acciones
        self.txtAcciones.setText(acciones)
        self.txtImporte.setText(importe)

        logging.error("Factor de conversion no siempre es 1")
        currency_conversion=1

        #Creamos un nuevo operinversiones 
        self.sim_op=self.investment.op.ObjectManager_copy_until_datetime(None, self.mem, self.investment)
        id_operinversiones=self.sim_op.get_highest_io_id ()+1##Para simular un id_operinversiones real, le asignamos uno
        if self.radDes.isChecked():#DESINVERSION
            d=InvestmentOperation(self.mem).init__create(self.mem.tiposoperaciones.find_by_id(5), datetime.datetime.now(pytz.timezone(self.mem.localzone.name)), self.investment, -acciones, impuestos, comision, valor_accion, "",  True, currency_conversion,  id_operinversiones)
        else:#REINVERSION
            d=InvestmentOperation(self.mem).init__create(self.mem.tiposoperaciones.find_by_id(4), datetime.datetime.now(pytz.timezone(self.mem.localzone.name)), self.investment, acciones, impuestos, comision, valor_accion, "",  True, currency_conversion,  id_operinversiones)
        self.sim_op.arr.append(d)

        (self.sim_opactual, self.sim_ophistorica)=self.sim_op.get_current_and_historical_operations()
        #After
        self.sim_op.myqtablewidget(self.tblOperaciones)
        self.sim_opactual.myqtablewidget(self.tblInvestmentsActualDespues, quote=self.investment.product.result.basic.last)
        self.sim_ophistorica.myqtablewidget(self.tblInvestmentsHistoricas)
        self.gains(self.tblGainsAfter,  self.investment.shares()+self.shares(), self.sim_opactual.average_price())
        
        #After at
        self.sim_opactual.myqtablewidget(self.tblInvestmentsActualDespuesAt, quote=at)
        
        #Before at
        self.investment.op_actual.myqtablewidget(self.tblInvestmentsActualAntesAt, quote=at)
        
        
        #Before
        self.tabAB.setCurrentIndex(1)
        self.tabOpAcHi.setCurrentIndex(1)
        self.gains(self.tblGainsBefore, self.investment.shares(), self.investment.op_actual.average_price())
        
        self.cmdOrder.setEnabled(True)
        self.cmdGraph.setEnabled(True)
                
    @pyqtSlot()
    def on_cmdGraph_released(self):
        self.investment.product.needStatus(2)
        d=QDialog(self)     
        d.setWindowTitle(self.tr("Reinvest graph"))
        d.showMaximized()
        lay = QVBoxLayout(d)
        
        wc=wdgProductHistoricalReinvestChart()
        wc.setProduct(self.investment.product, self.investment)
        wc.setReinvest( self.sim_op, self.sim_opactual)
        
        lay.addWidget(wc)
        wc.generate()
        wc.display()
        
        d.exec_()

    @pyqtSlot()
    def on_cmdOrder_released(self):
        d=QDialog(self)     
        d.setModal(True)
        d.setWindowTitle(self.tr("Add new order"))
        w=wdgOrdersAdd(self.mem, None, self.investment, d)
        w.txtShares.setText(self.txtAcciones.decimal())
        if self.radDes.isChecked():#DESINVERSION
            w.txtPrice.setText(-self.txtValorAccion.decimal())
        else:#REINVERSION
            w.txtPrice.setText(self.txtValorAccion.decimal())
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_()
        
    def gains(self, table,  shares,  averageprice):
        porcentages=[2.5, 5, 7.5, 10, 15, 30]
        table.applySettings()
        table.clearContents()
        table.setRowCount(len(porcentages))
        for i, tpc in enumerate(porcentages):        
            table.setItem(i, 0, Percentage(tpc, 100).qtablewidgetitem())
            tpcprice= averageprice*Decimal(1+tpc/100)
            table.setItem(i, 1, tpcprice.qtablewidgetitem())
            table.setItem(i, 2, ((tpcprice-averageprice)*shares).qtablewidgetitem())



from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import QWidget,  QDialog, QVBoxLayout, QMessageBox
from xulpymoney.ui.wdgSimulationsAdd import wdgSimulationsAdd
import xulpymoney.libdbupdates
import xulpymoney.ui.frmMain
from xulpymoney.ui.Ui_wdgSimulations import Ui_wdgSimulations
from xulpymoney.libxulpymoney import DBAdmin, SimulationManager, MemXulpymoney
from xulpymoney.connection_pg_qt import ConnectionQt
from xulpymoney.libxulpymoneyfunctions import qmessagebox
from xulpymoney.version import __versiondate__

class wdgSimulations(QWidget, Ui_wdgSimulations):
    def __init__(self, mem,  parent = None, name = None):
        QWidget.__init__(self,  parent)
        self.setupUi(self)
        self.resize(1000, 600)
        self.mem=mem
        self.parent=parent
        self.tblSimulations.settings(self.mem, "wdgSimulations")
        self.simulations=SimulationManager(self.mem)
        cur=self.mem.con.cursor()
        self.simulations.load_from_db(self.mem.con.mogrify("select * from simulations where database=%s order by creation",(self.mem.con.db, ) ), self.mem.con.db)
        cur.close()
        self.reload()
        self.mem_sim=None
        
    def reload(self):
        self.simulations.myqtablewidget(self.tblSimulations)

    def on_cmdCreate_released(self):
        d=QDialog(self)     
        d.setWindowTitle(self.tr("Create new simulation"))
        t=wdgSimulationsAdd(self.mem, d)
        lay = QVBoxLayout(d)
        lay.addWidget(t)
        d.exec_()
        if d.result()==QDialog.Accepted:
            self.simulations.append(t.simulation)
            self.reload()

    def on_tblSimulations_itemSelectionChanged(self):
        self.simulations.selected=None
        for i in self.tblSimulations.selectedItems():#itera por cada item no row.
            if i.column()==0:
                self.simulations.selected=self.simulations.arr[i.row()]
                
        if self.simulations.selected==None:
            self.cmdDelete.setEnabled(False)
            self.cmdConnect.setEnabled(False)
        else:
            self.cmdDelete.setEnabled(True)
            self.cmdConnect.setEnabled(True)
            
    def on_cmdDelete_released(self):
        reply = QMessageBox.question(self, 'Deleting a simulation', self.tr("Do you really want to delete this simulation?"), QMessageBox.Yes, QMessageBox.No)            
        if reply == QMessageBox.Yes:
            self.simulations.delete(self.simulations.selected)
            
            simcon=ConnectionQt().init__create(self.mem.con.user, self.mem.con.password, self.mem.con.server, self.mem.con.port, self.mem.con.db)
            simcon.connect()


            admin=DBAdmin(simcon)
            admin.drop_db(self.simulations.selected.simulated_db())
            self.mem.con.commit()
            simcon.commit()            
            self.reload()

    def on_cmdConnect_released(self):
        if not self.mem.con.is_superuser():
            qmessagebox(self.tr("The role of the user is not an administrator"))
            return
        simcon=ConnectionQt().init__create(self.mem.con.user, self.mem.con.password, self.mem.con.server, self.mem.con.port, self.simulations.selected.simulated_db())
        simcon.connect()
        self.mem_sim=MemXulpymoney()
        self.mem_sim.con=simcon
        
        ##Update database
        update=xulpymoney.libdbupdates.Update(self.mem_sim)
        if update.need_update()==True:
            update.run()
        
        self.mem_sim.frmMain = xulpymoney.ui.frmMain.frmMain(self.mem_sim)
        #No puedo visualizarlo, luego voy a usar un qdialog , ya que qmainwindow viene de qwidget.
        
        d=QDialog(self)        
        d.setStyleSheet("QDialog { background-color: rgb(255, 182, 182);  }");        
        d.setWindowTitle(self.tr("Xulpymoney SIMULATED IN {} 2010-{} \xa9").format(self.simulations.selected.simulated_db(),  __versiondate__.year))
        icon = QIcon()
        icon.addPixmap(QPixmap(":/xulpymoney/replication.png"), QIcon.Normal, QIcon.Off)
        d.setWindowIcon(icon)
        d.showMaximized()
        lay = QVBoxLayout(d)
        lay.addWidget(self.mem_sim.frmMain)
        d.exec_()
        



from PyQt5.QtCore import pyqtSignal, Qt
from PyQt5.QtGui import QIcon, QPixmap, QFont
from PyQt5.QtWidgets import QWidget, QDialog, QLabel, QLineEdit, QHBoxLayout, QToolButton, QVBoxLayout, QSizePolicy, QSpacerItem, QAbstractItemView
from xulpymoney.ui.myqtablewidget import myQTableWidget
from xulpymoney.libxulpymoney import ProductManager, ManagerSelectionMode
from xulpymoney.libxulpymoneyfunctions import qmessagebox

class wdgProductSelector(QWidget):
    """Para usarlo promocionar un qwidget en designer y darle los comportamientos de tamaña que neceseite
    incluso añadirlo a un layout."""
    selectionChanged=pyqtSignal()
    def __init__(self, parent):
        QWidget.__init__(self, parent)
        self.selected=None
    
    def setupUi(self, mem, investment=None):
        """Investement is used to set investment pointer. It's usefull to see investment data in product report"""
        self.mem=mem
        self.investment=investment#Optional
        
        self.horizontalLayout_2 = QHBoxLayout(self)
        self.horizontalLayout = QHBoxLayout()
        self.label = QLabel(self)
        self.label.setText(self.tr("Select a product"))
        self.horizontalLayout.addWidget(self.label)                                                                                                                                 
        self.txt = QLineEdit(self)                                                                                                                                       
        self.txt.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)                                                                             
        self.txt.setReadOnly(True)      
        self.txt.setToolTip(self.tr("Press the search button"))           
        self.horizontalLayout.addWidget(self.txt)                                                                                                                                 
        self.cmd= QToolButton(self)               
        icon = QIcon()
        icon.addPixmap(QPixmap(":/xulpymoney/document-preview-archive.png"), QIcon.Normal, QIcon.Off)
        self.cmd.setIcon(icon)                                                                                                                                   
        self.horizontalLayout.addWidget(self.cmd)                                                                                                                            
        self.horizontalLayout_2.addLayout(self.horizontalLayout)                
        self.cmd.released.connect(self.on_cmd_released)
        self.cmd.setToolTip(self.tr("Press to select a product"))
                                                                                                          
        self.cmdProduct= QToolButton(self)    
        icon2 = QIcon()
        icon2.addPixmap(QPixmap(":/xulpymoney/books.png"), QIcon.Normal, QIcon.Off)
        self.cmdProduct.setIcon(icon2)                                                                                                       
        self.horizontalLayout.addWidget(self.cmdProduct)  
        self.cmdProduct.setToolTip(self.tr("Press to see selected product information"))                    
        self.cmdProduct.released.connect(self.on_cmdProduct_released)
        self.cmdProduct.setEnabled(False)

    def on_cmd_released(self):
        d=frmProductSelector(self, self.mem)
        d.exec_()
        self.setSelected(d.products.selected)
        
    def on_cmdProduct_released(self):
        w=frmProductReport(self.mem, self.selected, self.investment,  self)
        w.exec_()
            
    def setSelected(self, product):
        """Recibe un objeto Product. No se usará posteriormente, por lo que puede no estar completo con get_basic.:."""
        self.selected=product
        if self.selected==None:
            self.txt.setText(self.tr("Not selected"))
            self.cmdProduct.setEnabled(False)     
            self.txt.setToolTip(self.tr("Press the search button"))                                                                                                                                                           
        else:      
            self.txt.setText("{0} ({1})".format(self.selected.name, self.selected.id))
            self.cmdProduct.setEnabled(True)
            self.txt.setToolTip(self.tr("Selected product"))    
            self.selectionChanged.emit()
        
    def showProductButton(self, boolean):
        if boolean==True:#Default
            self.cmdProduct.show()
        else:
            self.cmdProduct.hide()

class frmProductSelector(QDialog):
    def __init__(self, parent, mem):
        QDialog.__init__(self, parent)
        self.mem=mem
        self.products=ProductManager(self.mem)
        self.resize(1024, 500)
        self.horizontalLayout_2 = QHBoxLayout(self)
        self.verticalLayout = QVBoxLayout()
        self.lbl = QLabel(self)
        font = QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setAlignment(Qt.AlignCenter)
        self.verticalLayout.addWidget(self.lbl)
        self.horizontalLayout = QHBoxLayout()
        spacerItem = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.widget = wdgProductSelector(self)
        self.horizontalLayout.addWidget(self.widget)
        self.label = QLabel(self)
        self.horizontalLayout.addWidget(self.label)
        self.txt = QLineEdit(self)
        self.horizontalLayout.addWidget(self.txt)
        self.cmd = QToolButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap(":/xulpymoney/document-preview-archive.png"), QIcon.Normal, QIcon.Off)
        self.cmd.setIcon(icon)
        self.horizontalLayout.addWidget(self.cmd)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.tblInvestments = myQTableWidget(self)
        self.tblInvestments.setContextMenuPolicy(Qt.CustomContextMenu)
        self.tblInvestments.setAlternatingRowColors(True)
        self.tblInvestments.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.tblInvestments.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.tblInvestments.setSelectionMode(QAbstractItemView.SingleSelection)
        self.tblInvestments.settings(self.mem, "frmProductReport")

        self.tblInvestments.verticalHeader().setVisible(False)
        self.verticalLayout.addWidget(self.tblInvestments)
        self.lblFound = QLabel(self)
        self.verticalLayout.addWidget(self.lblFound)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.setWindowTitle(self.tr("Select a product"))
        self.lbl.setText(self.tr("Product list"))
        self.label.setText(self.tr("Search by code, ISIN, ticker or product name"))
        self.lblFound.setText(self.tr("Found registers"))

        self.setTabOrder(self.txt, self.cmd)
        self.setTabOrder(self.cmd, self.tblInvestments)
        self.cmd.released.connect(self.on_cmd_released)
        self.txt.returnPressed.connect(self.on_cmd_released)                    
        self.tblInvestments.itemSelectionChanged.connect(self.on_tblInvestments_itemSelectionChanged)
        self.tblInvestments.cellDoubleClicked.connect(self.on_tblInvestments_cellDoubleClicked)
        
    def on_cmd_released(self):
        if len(self.txt.text().upper())<=2:            
            qmessagebox(self.tr("Search too wide. You need more than 2 characters"))
            return
                
        self.products=self.mem.data.products.ProductManager_contains_string(self.txt.text().upper())
        self.products.setSelectionMode(ManagerSelectionMode.Object)
        self.products.needStatus(1, progress=True)
        self.products.order_by_name()
        self.lblFound.setText(self.tr("Found {0} registers").format(self.products.length()))
        self.products.myqtablewidget(self.tblInvestments)  
        
    def on_tblInvestments_cellDoubleClicked(self, row, column):
        self.done(0)
    
    def on_tblInvestments_itemSelectionChanged(self):
        self.products.selected=None
        for i in self.tblInvestments.selectedItems():
            if i.column()==0:
                self.products.selected=self.products.arr[i.row()]

from xulpymoney.ui.frmProductReport import frmProductReport



import argparse
from urllib.request import urlopen
from decimal import Decimal
import logging
import sys
from PyQt5.QtWidgets import QApplication
from xulpymoney.libxulpymoneyfunctions import addCommonToArgParse, addDebugSystem
from xulpymoney.libxulpymoney import MemSources

class CurrentPriceTicker:
    def __init__(self,ticker, xulpymoney, stockmarket):
        self.ticker=ticker
        self.stockmarket=stockmarket
        self.xulpymoney=xulpymoney
        self.dtaware=self.stockmarket.estimated_datetime_for_intraday_quote()
        self.price=None

    def __repr__(self):
        if self.xulpymoney!=None:
            return "PRICE | XULPYMONEY | {} | {} | {}".format(self.xulpymoney, self.dtaware, self.price)
        else:
            return "PRICE | STOCKMARKET | XX | TICKER | {} | {} | {}".format(self.ticker, self.dtaware , self.price)

    def get_price(self):
        try:
            web=urlopen('http://www.infobolsa.es/divisas')
        except:
            print ("ERROR | ERROR LOADING QUEFONDOS PAGE")
            sys.exit(255)

        if web==None:
            print ("ERROR | FETCHED EMPTY PAGE")
            sys.exit(255)

        web=web.read().decode('UTF-8')

        #Each currency data begins with <div class="currencyCont01">
        a=web.split("""<div class="currencyCont01">""")
        for block in a[1:]:
            if self.ticker=="EURUSD=X" and block.find("EUR/USD")!=-1:
                 self.price=Decimal(block.split("""<li class="txtCurren_ult top">""")[1].split("</li>")[0].replace(",","."))
                 return
        print ("ERROR | ERROR PARSING")
def main():
    app = QApplication(sys.argv)
    parser=argparse.ArgumentParser()
    parser.add_argument('--TICKER_XULPYMONEY', help='XULPYMONEY code', nargs=2, metavar="VALUE", required=True)
    parser.add_argument('--STOCKMARKET', help='Stock market id', metavar="ID", required=True)
    addCommonToArgParse(parser)
    args=parser.parse_args()        
    addDebugSystem(args)
    mem=MemSources()
    stockmarket=mem.stockmarkets.find_by_id(int(args.STOCKMARKET))
    logging.info(stockmarket)
    s=CurrentPriceTicker(args.TICKER_XULPYMONEY[0], args.TICKER_XULPYMONEY[1], stockmarket)
    s.get_price()
    print(s)




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgYear.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgYear(object):
    def setupUi(self, wdgYear):
        wdgYear.setObjectName("wdgYear")
        wdgYear.resize(448, 38)
        self.verticalLayout = QtWidgets.QVBoxLayout(wdgYear)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(wdgYear)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.cmdPrevious = QtWidgets.QToolButton(wdgYear)
        self.cmdPrevious.setObjectName("cmdPrevious")
        self.horizontalLayout.addWidget(self.cmdPrevious)
        self.cmbYear = QtWidgets.QComboBox(wdgYear)
        self.cmbYear.setObjectName("cmbYear")
        self.horizontalLayout.addWidget(self.cmbYear)
        self.cmdNext = QtWidgets.QToolButton(wdgYear)
        self.cmdNext.setObjectName("cmdNext")
        self.horizontalLayout.addWidget(self.cmdNext)
        self.line = QtWidgets.QFrame(wdgYear)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout.addWidget(self.line)
        self.cmdCurrent = QtWidgets.QPushButton(wdgYear)
        self.cmdCurrent.setObjectName("cmdCurrent")
        self.horizontalLayout.addWidget(self.cmdCurrent)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(wdgYear)
        QtCore.QMetaObject.connectSlotsByName(wdgYear)

    def retranslateUi(self, wdgYear):
        _translate = QtCore.QCoreApplication.translate
        self.label.setText(_translate("wdgYear", "Select a year"))
        self.cmdPrevious.setToolTip(_translate("wdgYear", "Previous year"))
        self.cmdPrevious.setText(_translate("wdgYear", "<"))
        self.cmbYear.setToolTip(_translate("wdgYear", "Select a year"))
        self.cmdNext.setToolTip(_translate("wdgYear", "Next year"))
        self.cmdNext.setText(_translate("wdgYear", ">"))
        self.cmdCurrent.setToolTip(_translate("wdgYear", "Current year"))
        self.cmdCurrent.setText(_translate("wdgYear", "Current Year"))





import datetime
from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QWidget
from xulpymoney.opportunities import Opportunity
from xulpymoney.libxulpymoneyfunctions import qmessagebox
from xulpymoney.libxulpymoneytypes import eInvestmentTypePosition
from xulpymoney.ui.Ui_wdgOpportunitiesAdd import Ui_wdgOpportunitiesAdd

class wdgOpportunitiesAdd(QWidget, Ui_wdgOpportunitiesAdd):
    def __init__(self, mem, opportunity=None,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.opportunity=opportunity
        self.parent=parent

        self.productSelector.setupUi(self.mem, None)
        if opportunity==None:
            self.lbl.setText("Add new opportunity")
            self.deDate.setDate(datetime.date.today())
            self.opportunity=Opportunity(self.mem)
            self.productSelector.setSelected(None)
            eInvestmentTypePosition.qcombobox(self.cmdInvesmentTypePosition)
        else:
            self.lbl.setText("Edit opportunity")
            self.deDate.setDate(self.opportunity.date)
            self.txtEntry.setText(self.opportunity.entry)
            self.txtTarget.setText(self.opportunity.target)
            self.txtStoploss.setText(self.opportunity.stoploss)
            self.productSelector.setSelected(self.opportunity.product)
            eInvestmentTypePosition.qcombobox(self.cmbInvestmentTypePosition, eInvestmentTypePosition.to_eInvestmentTypePosition(self.opportunity.short))

    @pyqtSlot()
    def on_buttonbox_accepted(self):
        if not (self.txtEntry.isValid()):
            qmessagebox(self.tr("Incorrect data. Try again."))
            return
        if self.productSelector.selected==None:
            qmessagebox(self.tr("You must select a product"))
            return

        self.opportunity.date=self.deDate.date().toPyDate()
        self.opportunity.entry=self.txtEntry.decimal()
        self.opportunity.target=self.txtTarget.decimal()
        self.opportunity.stoploss=self.txtStoploss.decimal()
        self.opportunity.product=self.productSelector.selected
        self.opportunity.short=eInvestmentTypePosition.to_boolean(self.cmbInvestmentTypePosition.itemData(self.cmbInvestmentTypePosition.currentIndex()))
        self.opportunity.save()
        self.mem.con.commit()
        self.parent.accept()

    @pyqtSlot()
    def on_buttonbox_rejected(self):
        self.parent.reject()
